<?php

namespace Drupal\webllm_ai\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Session\AccountInterface;
use Drupal\webllm_ai\Service\WebLlmService;
use Drupal\webllm_ai\Service\ModelManager;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Exception\BadRequestHttpException;
use Drupal\Core\Config\ConfigFactoryInterface;

/**
 * API controller for WebLLM AI endpoints.
 */
class ApiController extends ControllerBase {

  /**
   * The WebLLM service.
   *
   * @var \Drupal\webllm_ai\Service\WebLlmService
   */
  protected $webllmService;

  /**
   * The model manager.
   *
   * @var \Drupal\webllm_ai\Service\ModelManager
   */
  protected $modelManager;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Constructs an ApiController object.
   *
   * @param \Drupal\webllm_ai\Service\WebLlmService $webllm_service
   *   The WebLLM service.
   * @param \Drupal\webllm_ai\Service\ModelManager $model_manager
   *   The model manager.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   */
  public function __construct(WebLlmService $webllm_service, ModelManager $model_manager, ConfigFactoryInterface $config_factory) {
    $this->webllmService = $webllm_service;
    $this->modelManager = $model_manager;
    $this->configFactory = $config_factory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('webllm_ai.service'),
      $container->get('webllm_ai.model_manager'),
      $container->get('config.factory')
    );
  }

  /**
   * Handle chat completion requests.
   *
   * This endpoint is primarily for server-side integration and debugging.
   * The main WebLLM functionality runs client-side.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request object.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   The JSON response.
   */
  public function completion(Request $request) {
    $data = json_decode($request->getContent(), TRUE);

    if (!$data) {
      throw new BadRequestHttpException('Invalid JSON payload.');
    }

    // Validate required fields
    if (!isset($data['messages']) || !is_array($data['messages'])) {
      return new JsonResponse([
        'error' => 'messages field is required and must be an array',
      ], 400);
    }

    $model_id = $data['model'] ?? $this->webllmService->getDefaultModel();
    
    // Validate model
    if (!$this->webllmService->isValidModel($model_id)) {
      return new JsonResponse([
        'error' => 'Invalid or disallowed model: ' . $model_id,
        'available_models' => array_keys($this->webllmService->getAvailableModels()),
      ], 400);
    }

    // Since WebLLM runs client-side, we can't actually generate completions server-side
    // This endpoint serves as a validation and configuration helper
    $response_data = [
      'message' => 'WebLLM runs client-side. Use the JavaScript API for actual completions.',
      'validated' => TRUE,
      'model' => $model_id,
      'parameters' => [
        'temperature' => (float) ($data['temperature'] ?? $this->webllmService->getGenerationParameters()['temperature']),
        'max_tokens' => (int) ($data['max_tokens'] ?? $this->webllmService->getGenerationParameters()['max_tokens']),
      ],
      'system_prompt' => $this->webllmService->getDefaultSystemPrompt(),
      'browser_compatibility' => $this->webllmService->checkBrowserCompatibility($request->headers->get('User-Agent')),
    ];

    // Log the interaction intent
    if (!empty($data['messages'])) {
      $prompt = end($data['messages'])['content'] ?? '';
      $this->webllmService->logInteraction($model_id, $prompt, 'server-side-validation', [
        'endpoint' => 'completion',
        'user_agent' => $request->headers->get('User-Agent'),
        'ip' => $request->getClientIp(),
      ]);
    }

    return new JsonResponse($response_data);
  }

  /**
   * Get available models.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request object.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   The JSON response with available models.
   */
  public function getModels(Request $request) {
    $available_models = $this->webllmService->getAvailableModels();
    $all_models = $this->modelManager->getAllModels();
    
    $models_data = [];
    foreach ($available_models as $model_id => $model_info) {
      $full_info = $all_models[$model_id] ?? [];
      $models_data[] = [
        'id' => $model_id,
        'name' => $full_info['name'] ?? $model_info['name'] ?? $model_id,
        'description' => $full_info['description'] ?? $model_info['description'] ?? '',
        'family' => $full_info['family'] ?? 'unknown',
        'size' => $full_info['size'] ?? 'unknown',
        'performance_tier' => $full_info['performance_tier'] ?? 'medium',
        'recommended_memory_mb' => $full_info['recommended_memory'] ?? 0,
        'download_size_mb' => $full_info['download_size'] ?? 0,
        'context_length' => $full_info['context_length'] ?? 0,
        'languages' => $full_info['languages'] ?? ['en'],
        'capabilities' => $full_info['capabilities'] ?? [],
        'tags' => $full_info['tags'] ?? [],
      ];
    }

    // Get model recommendations based on request parameters
    $criteria = [];
    if ($request->query->has('max_memory_gb')) {
      $criteria['max_memory_mb'] = (int) $request->query->get('max_memory_gb') * 1024;
    }
    if ($request->query->has('language')) {
      $criteria['language'] = $request->query->get('language');
    }
    if ($request->query->has('performance_preference')) {
      $criteria['performance_preference'] = $request->query->get('performance_preference');
    }

    $recommendations = [];
    if (!empty($criteria)) {
      $recommended_models = $this->modelManager->getRecommendedModels($criteria);
      $recommendations = array_keys(array_slice($recommended_models, 0, 3, TRUE));
    }

    $response_data = [
      'models' => $models_data,
      'default_model' => $this->webllmService->getDefaultModel(),
      'recommendations' => $recommendations,
      'total_models' => count($models_data),
      'browser_compatibility' => $this->webllmService->checkBrowserCompatibility($request->headers->get('User-Agent')),
    ];

    return new JsonResponse($response_data);
  }

  /**
   * Get model advice based on user context.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request object.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   The JSON response with model advice.
   */
  public function getModelAdvice(Request $request) {
    $data = json_decode($request->getContent(), TRUE) ?: [];
    
    $context = [
      'available_memory_gb' => $data['memory_gb'] ?? NULL,
      'use_case' => $data['use_case'] ?? 'general',
      'language' => $data['language'] ?? 'en',
      'performance_preference' => $data['performance_preference'] ?? 'balanced',
    ];

    $advice = $this->modelManager->getModelAdvice($context);
    $model_info = $this->modelManager->getModel($advice['recommended_model']);

    $response_data = [
      'recommended_model' => $advice['recommended_model'],
      'model_info' => $model_info,
      'alternatives' => $advice['alternatives'],
      'warnings' => $advice['warnings'],
      'tips' => $advice['tips'],
      'context_used' => $context,
    ];

    return new JsonResponse($response_data);
  }

  /**
   * Get module configuration.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   JSON response with configuration.
   */
  public function getConfig() {
    $config = $this->configFactory->get('webllm_ai.settings');
    
    return new JsonResponse([
      'success' => TRUE,
      'config' => [
        'enabled' => $config->get('enabled'),
        'default_model' => $config->get('default_model'),
        'max_tokens' => $config->get('max_tokens'),
        'temperature' => $config->get('temperature'),
        'debug_mode' => $config->get('debug_mode'),
        'cache_enabled' => $config->get('cache_enabled'),
        'allowed_origins' => $config->get('allowed_origins'),
      ],
    ]);
  }

  /**
   * Check if WebGPU is supported.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   JSON response with WebGPU support status.
   */
  public function checkWebGpuSupport() {
    return new JsonResponse([
      'success' => TRUE,
      'message' => 'WebGPU support check should be done on client-side',
      'instructions' => 'Use navigator.gpu to check WebGPU availability in the browser',
    ]);
  }

  /**
   * Get model download progress.
   *
   * @param string $model_id
   *   The model ID.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   JSON response with download progress.
   */
  public function getModelProgress($model_id) {
    try {
      $progress = $this->modelManager->getModelProgress($model_id);
      return new JsonResponse([
        'success' => TRUE,
        'model_id' => $model_id,
        'progress' => $progress,
      ]);
    }
    catch (\Exception $e) {
      return new JsonResponse([
        'success' => FALSE,
        'error' => $e->getMessage(),
      ], Response::HTTP_NOT_FOUND);
    }
  }

  /**
   * Clear model cache.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   JSON response with cache clear status.
   */
  public function clearCache() {
    try {
      $this->modelManager->clearCache();
      return new JsonResponse([
        'success' => TRUE,
        'message' => 'Model cache cleared successfully',
      ]);
    }
    catch (\Exception $e) {
      return new JsonResponse([
        'success' => FALSE,
        'error' => $e->getMessage(),
      ], Response::HTTP_INTERNAL_SERVER_ERROR);
    }
  }

  /**
   * Health check endpoint.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   JSON response with health status.
   */
  public function healthCheck() {
    $config = $this->configFactory->get('webllm_ai.settings');
    
    return new JsonResponse([
      'success' => TRUE,
      'status' => 'healthy',
      'module_enabled' => $config->get('enabled'),
      'timestamp' => time(),
      'version' => '1.0.0',
    ]);
  }

  /**
   * Check access for API endpoints.
   *
   * @param \Drupal\Core\Session\AccountInterface $account
   *   The user account.
   *
   * @return \Drupal\Core\Access\AccessResultInterface
   *   The access result.
   */
  public function access(AccountInterface $account) {
    return AccessResult::allowedIfHasPermission($account, 'use webllm ai');
  }

}
