# WebLLM AI Drupal Module

A Drupal module that integrates [web-llm](https://github.com/mlc-ai/web-llm) to provide client-side AI inference capabilities without API dependencies.

## Features

- **Client-side AI inference** - Run Large Language Models directly in the browser using WebGPU
- **No API dependencies** - No need for OpenAI, Anthropic, or other external API keys
- **Privacy-focused** - All processing happens locally in the user's browser
- **Multiple model support** - Choose from various pre-trained models (Llama, Phi, Qwen, Gemma, Mistral)
- **Easy integration** - Blocks, templates, and API endpoints for seamless integration
- **Model management** - Download, cache, and manage AI models
- **Configurable interface** - Customizable chat interface with performance monitoring

## Requirements

- Drupal 9.0+ or Drupal 10.0+
- Modern browser with WebGPU support (Chrome 113+, Edge 113+, Firefox with experimental WebGPU enabled)
- Sufficient device memory (models range from 1GB to 40GB+)

## Installation

1. **Download the module** to your Drupal `modules/contrib` or `modules/custom` directory:
   ```bash
   cd web/modules/custom
   git clone [your-repo-url] webllm_ai
   ```

2. **Enable the module** using Drush or the admin interface:
   ```bash
   drush en webllm_ai
   ```

3. **Configure permissions** at `/admin/people/permissions`:
   - `access webllm ai demo` - Allow users to access the demo page
   - `access webllm ai api` - Allow access to API endpoints
   - `administer webllm ai` - Configure module settings

4. **Configure the module** at `/admin/config/system/webllm-ai`:
   - Enable the module
   - Set default model
   - Configure generation parameters (temperature, max tokens)
   - Enable debug mode if needed

## Usage

### Demo Page

Visit `/webllm-ai/demo` to try the full-featured chat interface with model selection and performance monitoring.

### Chat Block

1. Go to **Structure > Block layout** (`/admin/structure/block`)
2. Click **Place block** in your desired region
3. Find **Web-LLM AI Chat** and click **Place block**
4. Configure the block settings:
   - Default model
   - Interface height
   - Show/hide model selector
   - Show/hide performance stats
   - Customize placeholder text

### API Endpoints

The module provides several REST endpoints:

- `GET /webllm-ai/api/models` - Get available models
- `GET /webllm-ai/api/config` - Get module configuration
- `GET /webllm-ai/api/health` - Health check
- `GET /webllm-ai/api/model-progress/{model_id}` - Get model download progress
- `POST /webllm-ai/api/clear-cache` - Clear model cache

### Programmatic Usage

```php
// Get the WebLLM service
$webllm_service = \Drupal::service('webllm_ai.webllm_service');

// Get model manager
$model_manager = \Drupal::service('webllm_ai.model_manager');

// Get available models
$models = $model_manager->getAvailableModels();

// Create a chat block programmatically
$build = [
  '#theme' => 'webllm_ai_chat_block',
  '#configuration' => [
    'model' => 'Llama-3.2-3B-Instruct-q4f32_1-MLC',
    'height' => '400px',
    'show_model_selector' => TRUE,
    'placeholder_text' => 'Ask me anything...',
  ],
];
```

### JavaScript Integration

```javascript
// Access the WebLLM functionality
if (typeof drupalSettings.webllm_ai !== 'undefined') {
  const webllmConfig = drupalSettings.webllm_ai;
  
  // Initialize WebLLM (this happens automatically in the provided templates)
  // Your custom integration code here
}
```

## Available Models

The module supports various pre-trained models:

| Model | Size | Description |
|-------|------|-------------|
| Llama-3.2-3B-Instruct-q4f32_1-MLC | ~2GB | Fast, good for general chat |
| Llama-3.1-8B-Instruct-q4f32_1-MLC | ~5GB | Recommended balance of speed/quality |
| Phi-3.5-mini-instruct-q4f16_1-MLC | ~2GB | Lightweight, Microsoft's model |
| Qwen2.5-7B-Instruct-q4f16_1-MLC | ~4GB | Alibaba's multilingual model |
| Gemma-2-9B-It-q4f16_1-MLC | ~5GB | Google's Gemma model |

## Browser Compatibility

### WebGPU Support Required

| Browser | Version | Status |
|---------|---------|---------|
| Chrome | 113+ | ✅ Full support |
| Edge | 113+ | ✅ Full support |
| Firefox | 120+ | ⚠️ Experimental (enable `dom.webgpu.enabled`) |
| Safari | 18+ | ⚠️ Limited support |

### Enabling WebGPU in Firefox

1. Open `about:config`
2. Set `dom.webgpu.enabled` to `true`
3. Restart Firefox

## Configuration

### Module Settings (`/admin/config/system/webllm-ai`)

- **Enabled**: Enable/disable the module
- **Default Model**: Default model for new instances
- **Max Tokens**: Maximum tokens per generation
- **Temperature**: Control randomness (0.0 = deterministic, 2.0 = very random)
- **Debug Mode**: Show debug information
- **Cache Enabled**: Enable model caching
- **Allowed Origins**: CORS configuration for API endpoints

### Performance Considerations

- **Memory Usage**: Models require significant RAM (2-40GB+)
- **Download Time**: First model download can take several minutes
- **Caching**: Models are cached in browser storage for subsequent visits
- **GPU Requirements**: WebGPU-compatible graphics card recommended

## Troubleshooting

### WebGPU Not Available

**Error**: "WebGPU is not supported in this browser"

**Solutions**:
- Update to a compatible browser version
- Enable WebGPU in Firefox (see above)
- Check hardware compatibility

### Model Download Fails

**Error**: Model download stuck or fails

**Solutions**:
- Check internet connection
- Clear browser cache
- Try a smaller model first
- Check browser console for errors

### Out of Memory

**Error**: "Out of memory" during model loading

**Solutions**:
- Try a smaller model (3B instead of 8B)
- Close other tabs/applications
- Use a device with more RAM

### Performance Issues

**Symptoms**: Slow token generation

**Solutions**:
- Reduce max_tokens setting
- Lower temperature setting
- Use a smaller model
- Check GPU drivers are updated

## Development

### File Structure

```
webllm_ai/
├── config/
│   ├── install/
│   │   └── webllm_ai.settings.yml
│   └── schema/
│       └── webllm_ai.schema.yml
├── css/
│   └── webllm-ai.css
├── js/
│   ├── webllm-ai.js
│   └── webllm-admin.js
├── src/
│   ├── Controller/
│   │   ├── ApiController.php
│   │   └── WebLLMController.php
│   ├── Form/
│   │   └── SettingsForm.php
│   ├── Plugin/
│   │   └── Block/
│   │       └── WebLLMChatBlock.php
│   └── Service/
│       ├── ModelManagerService.php
│       └── WebLLMService.php
├── templates/
│   ├── webllm-ai-demo.html.twig
│   ├── webllm-ai-models.html.twig
│   └── webllm-ai-chat-block.html.twig
├── webllm_ai.info.yml
├── webllm_ai.libraries.yml
├── webllm_ai.module
├── webllm_ai.permissions.yml
├── webllm_ai.routing.yml
├── webllm_ai.services.yml
└── README.md
```

### Contributing

1. Fork the repository
2. Create a feature branch
3. Make your changes
4. Test thoroughly
5. Submit a pull request

## Security Considerations

- All AI processing happens client-side
- No data is sent to external servers
- Models are downloaded from trusted sources
- CORS settings can be configured for API access

## License

This module is licensed under the GPL-2.0+ license, same as Drupal core.

## Support

For issues, feature requests, or questions:

1. Check the [troubleshooting section](#troubleshooting)
2. Search existing issues
3. Create a new issue with detailed information

## Changelog

### 1.0.0
- Initial release
- Basic chat interface
- Model management
- Block integration
- API endpoints
- Admin configuration
