/**
 * @file
 * RAG Demo - Simple WebLLM Integration with Content Extraction
 * Based on the working webllm_simple.js pattern
 */

console.log('WebLLM RAG Demo Loading...');

let webllmEngine = null;
let isModelLoaded = false;
let extractedContent = '';

// Simple status update function
function updateStatus(message) {
    console.log('Status:', message);
    const statusElement = document.querySelector('.status-text');
    if (statusElement) {
        statusElement.textContent = message;
    }

    // Also update any status div if it exists
    const statusDiv = document.getElementById('status-messages');
    if (statusDiv) {
        statusDiv.innerHTML = `<div class="alert alert-info">${message}</div>`;
    }
}

// Extract content from current page
function extractPageContent() {
    console.log('🔍 Extracting page content...');
    updateStatus('Extracting page content...');

    let content = '';

    // First try to get content from demo sample content
    const sampleContent = document.querySelector('.demo-sample-content');
    if (sampleContent) {
        content = sampleContent.innerText || sampleContent.textContent || '';
        console.log('📄 Found sample content:', content.length, 'characters');
    }

    // Fallback to main content areas
    if (!content || content.length < 100) {
        const selectors = ['main', '.main-content', '.content', 'article', '#content', '.page-content'];
        for (const selector of selectors) {
            const element = document.querySelector(selector);
            if (element) {
                content = element.innerText || element.textContent || '';
                if (content.length > 100) {
                    console.log('📄 Using content from:', selector);
                    break;
                }
            }
        }
    }

    // Final fallback to body content
    if (!content || content.length < 100) {
        console.log('⚠️ Using body content as fallback');
        content = document.body.innerText || document.body.textContent || '';
    }

    // Clean content
    content = content.replace(/\s+/g, ' ').trim();

    if (content.length > 50) {
        extractedContent = content;
        console.log('✅ Content extracted:', content.length, 'characters');
        console.log('📄 Content preview:', content.substring(0, 200) + '...');

        // Update UI
        const contentDisplay = document.getElementById('extracted-content');
        if (contentDisplay) {
            contentDisplay.value = content.substring(0, 1000) + (content.length > 1000 ? '...' : '');
        }

        const wordCount = document.getElementById('word-count');
        if (wordCount) wordCount.textContent = content.split(' ').length;

        const charCount = document.getElementById('char-count');
        if (charCount) charCount.textContent = content.length;

        const sourceUrl = document.getElementById('source-url');
        if (sourceUrl) sourceUrl.textContent = window.location.href;

        // Show next sections
        const contentSection = document.querySelector('.content-display-section');
        if (contentSection) contentSection.style.display = 'block';

        const modelSection = document.querySelector('.model-selection-section');
        if (modelSection) modelSection.style.display = 'block';

        updateStatus('Content extracted successfully! Now load the AI model.');
        return true;
    } else {
        console.error('❌ No sufficient content found');
        updateStatus('Error: No readable content found on this page.');
        return false;
    }
}

// Check WebGPU support
function checkWebGPUSupport() {
    if ('gpu' in navigator) {
        console.log('✓ WebGPU supported');
        return true;
    } else {
        console.log('✗ WebGPU not supported');
        updateStatus('WebGPU not supported - some features may not work');
        return false;
    }
}

// Load the lightweight model for testing
async function loadSimpleModel() {
    const loadBtn = document.getElementById('load-model-btn');
    const progressDiv = document.getElementById('loading-progress');
    const progressBar = document.getElementById('progress-bar');
    const progressText = document.getElementById('progress-text');

    try {
        // Use lightweight model for testing
        const modelId = "Phi-3.5-mini-instruct-q4f16_1-MLC";

        console.log('🔄 Loading model:', modelId);
        updateStatus('Loading model: ' + modelId + '. This may take several minutes...');

        // Show progress
        if (progressDiv) progressDiv.style.display = 'block';
        if (loadBtn) {
            loadBtn.disabled = true;
            loadBtn.textContent = 'Loading...';
        }

        // Progress callback
        const progressCallback = (report) => {
            console.log('📊 Progress:', report);
            if (report.progress !== undefined) {
                const percentage = Math.round(report.progress * 100);
                if (progressBar) progressBar.style.width = percentage + '%';
                if (progressText) progressText.textContent = percentage + '%';
                updateStatus(`Loading model: ${percentage}%`);
            }
            if (report.text) {
                updateStatus(`Loading: ${report.text}`);
            }
        };

        // Initialize WebLLM engine if not already done
        if (!webllmEngine) {
            console.log('🚀 Initializing WebLLM engine...');
            const webllm = await import('https://esm.run/@mlc-ai/web-llm');
            webllmEngine = await webllm.CreateMLCEngine(modelId, {
                initProgressCallback: progressCallback
            });
        } else {
            // Reload with new model
            await webllmEngine.reload(modelId, undefined, {
                initProgressCallback: progressCallback
            });
        }

        // Success!
        isModelLoaded = true;
        console.log('✅ Model loaded successfully!');
        updateStatus('Model ready! You can now ask questions about the content.');

        if (loadBtn) {
            loadBtn.textContent = 'Model Loaded ✓';
            loadBtn.style.background = '#4caf50';
        }
        if (progressDiv) progressDiv.style.display = 'none';

        // Enable chat
        enableChat();

    } catch (error) {
        console.error('❌ Error loading model:', error);
        updateStatus('Error loading model: ' + error.message);

        if (loadBtn) {
            loadBtn.disabled = false;
            loadBtn.textContent = 'Retry Loading Model';
        }
        if (progressDiv) progressDiv.style.display = 'none';
    }
}

// Enable chat interface
function enableChat() {
    const chatInput = document.getElementById('question-input');
    const sendBtn = document.getElementById('ask-question-btn');

    if (chatInput) {
        chatInput.disabled = false;
        chatInput.placeholder = 'Ask a question about the extracted content...';
    }

    if (sendBtn) {
        sendBtn.disabled = false;
    }

    // Show chat section
    const chatSection = document.querySelector('.chat-section');
    if (chatSection) chatSection.style.display = 'block';
}

// Send message with RAG context
async function sendMessage() {
    const chatInput = document.getElementById('question-input');
    const chatMessages = document.getElementById('chat-messages');

    if (!chatInput || !isModelLoaded || !extractedContent) {
        console.error('❌ Missing requirements for sending message');
        updateStatus('Please make sure you have extracted content and loaded the model first.');
        return;
    }

    const userMessage = chatInput.value.trim();
    if (!userMessage) {
        updateStatus('Please enter a question.');
        return;
    }

    try {
        // Add user message to chat
        addMessageToChat('user', userMessage);
        chatInput.value = '';

        // Add thinking message
        const thinkingMsg = addMessageToChat('assistant', 'Thinking...');

        // Create RAG prompt
        const ragPrompt = `Based on the following content from the webpage:

${extractedContent}

Please answer this question: ${userMessage}

Answer based only on the information provided in the content above. If the answer is not found in the content, please say so.`;

        console.log('🤖 Generating response...');

        // Generate response
        const response = await webllmEngine.chat.completions.create({
            messages: [{ role: "user", content: ragPrompt }],
            temperature: 0.7,
            max_tokens: 1000,
        });

        const assistantMessage = response.choices[0].message.content;

        // Update thinking message with response
        updateChatMessage(thinkingMsg, assistantMessage);

        console.log('✅ Response generated');

    } catch (error) {
        console.error('❌ Error generating response:', error);
        updateStatus('Error generating response: ' + error.message);
    }
}

// Add message to chat
function addMessageToChat(role, content) {
    const chatMessages = document.getElementById('chat-messages');
    if (!chatMessages) return null;

    const messageId = 'msg-' + Date.now() + '-' + Math.random().toString(36).substr(2, 9);
    const messageClass = role === 'user' ? 'user-message' : 'assistant-message';

    const messageDiv = document.createElement('div');
    messageDiv.id = messageId;
    messageDiv.className = `chat-message ${messageClass}`;
    messageDiv.innerHTML = `
        <div class="message-role">${role === 'user' ? 'You' : 'AI Assistant'}</div>
        <div class="message-content">${escapeHtml(content)}</div>
        <div class="message-time">${new Date().toLocaleTimeString()}</div>
    `;

    chatMessages.appendChild(messageDiv);
    chatMessages.scrollTop = chatMessages.scrollHeight;

    return messageDiv;
}

// Update chat message content
function updateChatMessage(messageElement, content) {
    if (messageElement) {
        const contentDiv = messageElement.querySelector('.message-content');
        if (contentDiv) {
            contentDiv.innerHTML = escapeHtml(content);
        }
    }
}

// Escape HTML
function escapeHtml(text) {
    const div = document.createElement('div');
    div.textContent = text;
    return div.innerHTML;
}

// Initialize when DOM is ready
document.addEventListener('DOMContentLoaded', function () {
    console.log('🚀 RAG Demo initializing...');

    // Initialize page info
    const pageTitle = document.getElementById('current-page-title');
    const pageUrl = document.getElementById('current-page-url');

    if (pageTitle) pageTitle.textContent = document.title || 'Untitled Page';
    if (pageUrl) pageUrl.textContent = window.location.href;

    // Check WebGPU support
    checkWebGPUSupport();

    updateStatus('Ready! Click "Extract Page Content" to begin.');

    // Bind extract content button
    const extractBtn = document.getElementById('extract-content-btn');
    if (extractBtn) {
        console.log('📍 Found extract button, binding click handler');
        extractBtn.addEventListener('click', function (e) {
            e.preventDefault();
            console.log('🎯 Extract button clicked');
            extractPageContent();
        });
    } else {
        console.error('❌ Extract button not found!');
    }

    // Bind load model button
    const loadModelBtn = document.getElementById('load-model-btn');
    if (loadModelBtn) {
        console.log('📍 Found load model button, binding click handler');
        loadModelBtn.addEventListener('click', function (e) {
            e.preventDefault();
            console.log('🎯 Load model button clicked');
            loadSimpleModel();
        });
    } else {
        console.error('❌ Load model button not found!');
    }

    // Bind ask question button
    const askBtn = document.getElementById('ask-question-btn');
    if (askBtn) {
        console.log('📍 Found ask question button, binding click handler');
        askBtn.addEventListener('click', function (e) {
            e.preventDefault();
            console.log('🎯 Ask question button clicked');
            sendMessage();
        });
    }

    // Enter key for question input
    const questionInput = document.getElementById('question-input');
    if (questionInput) {
        questionInput.addEventListener('keypress', function (e) {
            if (e.key === 'Enter' && !e.shiftKey) {
                e.preventDefault();
                sendMessage();
            }
        });
    }

    console.log('✅ RAG Demo ready');

    // Debug: List all buttons found on the page
    const allButtons = document.querySelectorAll('button');
    console.log('📍 All buttons found:', Array.from(allButtons).map(btn => ({ id: btn.id, text: btn.textContent })));
});

// Also bind when the page loads (backup)
window.addEventListener('load', function () {
    console.log('🔄 Window loaded, re-checking button bindings...');

    const extractBtn = document.getElementById('extract-content-btn');
    if (extractBtn && !extractBtn.hasAttribute('data-bound')) {
        extractBtn.setAttribute('data-bound', 'true');
        extractBtn.addEventListener('click', function (e) {
            e.preventDefault();
            console.log('🎯 Extract button clicked (backup binding)');
            extractPageContent();
        });
        console.log('✅ Backup binding applied to extract button');
    }
});
