/**
 * @file
 * Admin functionality for Web-LLM AI module.
 */

(function ($, Drupal, drupalSettings) {
    'use strict';

    /**
     * Web-LLM Admin functionality.
     */
    Drupal.behaviors.webllmAdmin = {
        attach: function (context, settings) {
            const $modelsGrid = $('#models-grid', context);
            const $refreshBtn = $('#refresh-models', context);
            const $clearCacheBtn = $('#clear-cache', context);

            if ($modelsGrid.length) {
                this.initializeModelManagement($modelsGrid);
            }

            // Refresh models button
            $refreshBtn.once('webllm-refresh').on('click', function (e) {
                e.preventDefault();
                location.reload();
            });

            // Clear cache button
            $clearCacheBtn.once('webllm-clear-cache').on('click', function (e) {
                e.preventDefault();
                if (confirm(Drupal.t('Are you sure you want to clear the model cache? This will remove all downloaded models.'))) {
                    fetch('/webllm-ai/api/clear-cache', {
                        method: 'POST',
                        headers: {
                            'Content-Type': 'application/json',
                        },
                    })
                        .then(response => response.json())
                        .then(data => {
                            if (data.success) {
                                Drupal.announce(Drupal.t('Model cache cleared successfully.'));
                                location.reload();
                            } else {
                                console.error('Failed to clear cache:', data.error);
                                alert(Drupal.t('Failed to clear cache: @error', { '@error': data.error }));
                            }
                        })
                        .catch(error => {
                            console.error('Error clearing cache:', error);
                            alert(Drupal.t('Error clearing cache. Please try again.'));
                        });
                }
            });
        },

        /**
         * Initialize model management functionality.
         */
        initializeModelManagement: function ($grid) {
            const self = this;

            // Download model buttons
            $grid.find('.download-model').once('webllm-download').on('click', function (e) {
                e.preventDefault();
                const modelId = $(this).data('model');
                const $card = $(this).closest('.model-card');
                self.downloadModel(modelId, $card);
            });

            // Test model buttons
            $grid.find('.test-model').once('webllm-test').on('click', function (e) {
                e.preventDefault();
                const modelId = $(this).data('model');
                self.testModel(modelId);
            });

            // Check model status on page load
            this.checkModelStatus($grid);
        },

        /**
         * Download a model.
         */
        downloadModel: function (modelId, $card) {
            const $downloadBtn = $card.find('.download-model');
            const $testBtn = $card.find('.test-model');
            const $progress = $card.find('.model-progress');
            const $progressFill = $progress.find('.progress-fill');
            const $progressText = $progress.find('.progress-text');
            const $status = $card.find('.status-badge');

            // Show progress and disable button
            $downloadBtn.prop('disabled', true).text(Drupal.t('Downloading...'));
            $progress.show();
            $status.removeClass().addClass('status-badge status-downloading').text(Drupal.t('Downloading'));

            // Simulate download progress (in real implementation, this would come from web-llm)
            let progress = 0;
            const progressInterval = setInterval(() => {
                progress += Math.random() * 10;
                if (progress > 100) progress = 100;

                $progressFill.css('width', progress + '%');
                $progressText.text(Math.round(progress) + '%');

                if (progress >= 100) {
                    clearInterval(progressInterval);

                    // Download complete
                    setTimeout(() => {
                        $progress.hide();
                        $downloadBtn.hide();
                        $testBtn.show();
                        $status.removeClass().addClass('status-badge status-ready').text(Drupal.t('Ready'));

                        Drupal.announce(Drupal.t('Model @model downloaded successfully.', { '@model': modelId }));
                    }, 500);
                }
            }, 200);
        },

        /**
         * Test a model.
         */
        testModel: function (modelId) {
            // Open demo page in new tab with the specific model
            const demoUrl = '/webllm-ai/demo?model=' + encodeURIComponent(modelId);
            window.open(demoUrl, '_blank');
        },

        /**
         * Check status of all models.
         */
        checkModelStatus: function ($grid) {
            // In a real implementation, this would check which models are already downloaded
            // For now, we'll assume all models need to be downloaded
            $grid.find('.model-card').each(function () {
                const $card = $(this);
                const $status = $card.find('.status-badge');
                const $downloadBtn = $card.find('.download-model');
                const $testBtn = $card.find('.test-model');

                // Check if model is already downloaded (this would be real logic)
                const isDownloaded = false; // Placeholder

                if (isDownloaded) {
                    $downloadBtn.hide();
                    $testBtn.show();
                    $status.removeClass().addClass('status-badge status-ready').text(Drupal.t('Ready'));
                } else {
                    $downloadBtn.show();
                    $testBtn.hide();
                    $status.removeClass().addClass('status-badge status-not-downloaded').text(Drupal.t('Not Downloaded'));
                }
            });
        }
    };

})(jQuery, Drupal, drupalSettings);
