/**
 * @file
 * WebLLM AI JavaScript functionality.
 */

(function (Drupal, drupalSettings) {
    'use strict';

    // Main WebLLM AI namespace
    Drupal.webllmAI = Drupal.webllmAI || {};

    /**
     * WebLLM Engine Manager
     */
    Drupal.webllmAI.EngineManager = class {
        constructor() {
            this.engine = null;
            this.isLoading = false;
            this.isInitialized = false;
            this.currentModel = null;
            this.progressCallback = null;
            this.chatHistory = [];
        }

        /**
         * Initialize the WebLLM engine with a specific model
         */
        async initializeEngine(modelId, progressCallback) {
            if (this.isLoading) {
                throw new Error('Engine is already loading');
            }

            this.isLoading = true;
            this.progressCallback = progressCallback;

            try {
                // Import WebLLM dynamically
                const webllm = await import('https://esm.run/@mlc-ai/web-llm');

                // Create engine with progress callback
                this.engine = await webllm.CreateMLCEngine(
                    modelId,
                    {
                        initProgressCallback: (progress) => {
                            if (this.progressCallback) {
                                this.progressCallback(progress);
                            }
                        }
                    }
                );

                this.currentModel = modelId;
                this.isInitialized = true;
                this.isLoading = false;

                return true;
            } catch (error) {
                this.isLoading = false;
                console.error('Failed to initialize WebLLM engine:', error);
                throw error;
            }
        }

        /**
         * Generate chat completion
         */
        async generateCompletion(messages, options = {}) {
            if (!this.isInitialized || !this.engine) {
                throw new Error('Engine not initialized');
            }

            try {
                const defaultOptions = {
                    temperature: 0.7,
                    max_tokens: 1000,
                    stream: false
                };

                const completionOptions = { ...defaultOptions, ...options, messages };

                if (completionOptions.stream) {
                    return await this.engine.chat.completions.create(completionOptions);
                } else {
                    const reply = await this.engine.chat.completions.create(completionOptions);
                    return reply;
                }
            } catch (error) {
                console.error('Failed to generate completion:', error);
                throw error;
            }
        }

        /**
         * Add message to chat history
         */
        addToHistory(role, content) {
            this.chatHistory.push({ role, content });
        }

        /**
         * Get chat history
         */
        getHistory() {
            return this.chatHistory;
        }

        /**
         * Clear chat history
         */
        clearHistory() {
            this.chatHistory = [];
        }

        /**
         * Check if engine is ready
         */
        isReady() {
            return this.isInitialized && !this.isLoading;
        }        /**
         * Get available models from API
         */
        async getAvailableModels() {
            try {
                const response = await fetch('/webllm-ai/api/models');
                const data = await response.json();

                if (data.models && Array.isArray(data.models)) {
                    return data.models;
                }

                // Fallback to static list
                return this.getStaticModels();
            } catch (error) {
                console.error('Failed to fetch models from API:', error);
                // Fallback to static list
                return this.getStaticModels();
            }
        }

        /**
         * Get static fallback models
         */
        getStaticModels() {
            return [
                {
                    id: 'Llama-3.1-8B-Instruct-q4f32_1-MLC',
                    name: 'Llama 3.1 8B Instruct',
                    description: 'Meta\'s Llama 3.1 8B model optimized for instruction following',
                    size: '8B'
                },
                {
                    id: 'Phi-3.5-mini-instruct-q4f16_1-MLC',
                    name: 'Phi 3.5 Mini Instruct',
                    description: 'Microsoft\'s compact and efficient Phi model',
                    size: '3.8B'
                },
                {
                    id: 'Qwen2.5-7B-Instruct-q4f16_1-MLC',
                    name: 'Qwen 2.5 7B Instruct',
                    description: 'Alibaba\'s multilingual Qwen model',
                    size: '7B'
                }
            ];
        }
    };

    /**
     * Chat Interface Handler
     */
    Drupal.webllmAI.ChatInterface = class {
        constructor(containerId, engineManager) {
            this.container = document.getElementById(containerId);
            this.engineManager = engineManager;
            this.elements = {};
            this.isStreaming = false;

            this.init();
        } async init() {
            if (!this.container) {
                console.error('Chat container not found');
                return;
            }

            await this.createElements();
            this.attachEventListeners();
            this.updateUI();
        }

        async createElements() {
            // Show loading state first
            this.container.innerHTML = `
        <div class="webllm-settings">
          <h3>WebLLM AI Chat</h3>
          <div class="webllm-controls">
            <div class="webllm-setting-group webllm-model-select">
              <label for="webllm-model">Select Model:</label>
              <select id="webllm-model">
                <option value="">Loading models...</option>
              </select>
            </div>
            <button id="webllm-load-model" class="webllm-send-btn" disabled>Load Model</button>
          </div>
          <div id="webllm-progress-container" class="webllm-hidden">
            <div class="webllm-progress">
              <div id="webllm-progress-bar" class="webllm-progress-bar" style="width: 0%"></div>
            </div>
            <div id="webllm-progress-text" class="webllm-progress-text">Loading...</div>
          </div>
        </div>
        
        <div id="webllm-messages" class="webllm-messages">
          <div class="webllm-message system">
            Select a model and click "Load Model" to start chatting. 
            Note: First-time model loading may take several minutes as models are downloaded and cached.
          </div>
        </div>
        
        <div class="webllm-input-area">
          <textarea id="webllm-input" class="webllm-input" 
                    placeholder="Type your message here..." 
                    rows="3" disabled></textarea>
          <button id="webllm-send" class="webllm-send-btn" disabled>Send</button>
        </div>
      `;

            // Store element references
            this.elements = {
                modelSelect: document.getElementById('webllm-model'),
                loadModelBtn: document.getElementById('webllm-load-model'),
                progressContainer: document.getElementById('webllm-progress-container'),
                progressBar: document.getElementById('webllm-progress-bar'),
                progressText: document.getElementById('webllm-progress-text'),
                messages: document.getElementById('webllm-messages'),
                input: document.getElementById('webllm-input'),
                sendBtn: document.getElementById('webllm-send')
            };

            // Load models from API
            await this.loadAvailableModels();
        }

        async loadAvailableModels() {
            try {
                const models = await this.engineManager.getAvailableModels();
                const modelOptions = models.map(model => {
                    const name = model.name || model.id;
                    const sizeInfo = model.size ? ` (${model.size})` : '';
                    return `<option value="${model.id}">${name}${sizeInfo}</option>`;
                }).join('');

                this.elements.modelSelect.innerHTML = `
                    <option value="">Choose a model...</option>
                    ${modelOptions}
                `;
            } catch (error) {
                console.error('Failed to load models:', error);
                this.elements.modelSelect.innerHTML = `
                    <option value="">Error loading models</option>
                `;
            }
        }

        attachEventListeners() {
            // Model selection
            this.elements.modelSelect.addEventListener('change', () => {
                this.elements.loadModelBtn.disabled = !this.elements.modelSelect.value;
            });

            // Load model button
            this.elements.loadModelBtn.addEventListener('click', () => {
                this.loadModel();
            });

            // Send message
            this.elements.sendBtn.addEventListener('click', () => {
                this.sendMessage();
            });

            // Enter key to send (Ctrl+Enter for new line)
            this.elements.input.addEventListener('keydown', (e) => {
                if (e.key === 'Enter' && !e.ctrlKey && !e.shiftKey) {
                    e.preventDefault();
                    this.sendMessage();
                }
            });
        }

        async loadModel() {
            const selectedModel = this.elements.modelSelect.value;
            if (!selectedModel) return;

            try {
                this.showProgress(true);
                this.elements.loadModelBtn.disabled = true;
                this.elements.modelSelect.disabled = true;

                await this.engineManager.initializeEngine(selectedModel, (progress) => {
                    this.updateProgress(progress);
                });

                this.showProgress(false);
                this.addMessage('system', `Model "${selectedModel}" loaded successfully! You can now start chatting.`);
                this.updateUI();

            } catch (error) {
                this.showProgress(false);
                this.addMessage('system', `Error loading model: ${error.message}`, 'error');
                this.elements.loadModelBtn.disabled = false;
                this.elements.modelSelect.disabled = false;
            }
        }

        async sendMessage() {
            const message = this.elements.input.value.trim();
            if (!message || !this.engineManager.isReady()) return;

            try {
                // Add user message
                this.addMessage('user', message);
                this.engineManager.addToHistory('user', message);
                this.elements.input.value = '';
                this.updateUI();

                // Show loading
                const loadingDiv = this.addMessage('assistant', '', 'loading');

                // Get system prompt from settings
                const systemPrompt = drupalSettings.webllmAI?.systemPrompt ||
                    'You are a helpful AI assistant integrated into a Drupal website.';

                // Prepare messages
                const messages = [
                    { role: 'system', content: systemPrompt },
                    ...this.engineManager.getHistory()
                ];

                // Generate response
                const response = await this.engineManager.generateCompletion(messages, {
                    temperature: parseFloat(drupalSettings.webllmAI?.temperature || 0.7),
                    max_tokens: parseInt(drupalSettings.webllmAI?.maxTokens || 1000)
                });

                // Remove loading message
                loadingDiv.remove();

                // Add assistant response
                const assistantMessage = response.choices[0].message.content;
                this.addMessage('assistant', assistantMessage);
                this.engineManager.addToHistory('assistant', assistantMessage);

            } catch (error) {
                console.error('Error sending message:', error);
                this.addMessage('system', `Error: ${error.message}`, 'error');
            }

            this.updateUI();
        }

        addMessage(role, content, type = null) {
            const messageDiv = document.createElement('div');
            messageDiv.className = `webllm-message ${role}`;

            if (type === 'error') {
                messageDiv.classList.add('webllm-error');
            } else if (type === 'loading') {
                messageDiv.innerHTML = '<span class="webllm-loading">AI is thinking</span>';
                this.elements.messages.appendChild(messageDiv);
                this.elements.messages.scrollTop = this.elements.messages.scrollHeight;
                return messageDiv;
            }

            messageDiv.textContent = content;
            messageDiv.classList.add('webllm-fade-in');

            this.elements.messages.appendChild(messageDiv);
            this.elements.messages.scrollTop = this.elements.messages.scrollHeight;

            return messageDiv;
        }

        showProgress(show) {
            if (show) {
                this.elements.progressContainer.classList.remove('webllm-hidden');
            } else {
                this.elements.progressContainer.classList.add('webllm-hidden');
            }
        }

        updateProgress(progress) {
            if (progress.progress !== undefined) {
                const percentage = Math.round(progress.progress * 100);
                this.elements.progressBar.style.width = `${percentage}%`;
            }

            if (progress.text) {
                this.elements.progressText.textContent = progress.text;
            }
        }

        updateUI() {
            const isReady = this.engineManager.isReady();
            this.elements.input.disabled = !isReady;
            this.elements.sendBtn.disabled = !isReady || this.isStreaming;

            if (!isReady && !this.engineManager.isLoading) {
                this.elements.loadModelBtn.disabled = !this.elements.modelSelect.value;
                this.elements.modelSelect.disabled = false;
            }
        }
    };

    /**
     * Initialize WebLLM AI when the page loads
     */    Drupal.behaviors.webllmAI = {
        attach: function (context, settings) {
            // Initialize chat interfaces - look for our demo template
            $(context).find('.webllm-ai-demo').once('webllm-ai').each(function () {
                const container = this;
                const containerId = container.id || 'webllm-chat-' + Math.random().toString(36).substr(2, 9);
                container.id = containerId;

                // Create engine manager
                const engineManager = new Drupal.webllmAI.EngineManager();

                // Create chat interface
                const chatInterface = new Drupal.webllmAI.ChatInterface(containerId, engineManager);

                // Store reference for potential API access
                container.webllmInterface = chatInterface;
            });

            // Also initialize generic chat containers
            $(context).find('.webllm-chat-container').once('webllm-ai').each(function () {
                const container = this;
                const containerId = container.id || 'webllm-chat-' + Math.random().toString(36).substr(2, 9);
                container.id = containerId;

                // Create engine manager
                const engineManager = new Drupal.webllmAI.EngineManager();

                // Create chat interface
                const chatInterface = new Drupal.webllmAI.ChatInterface(containerId, engineManager);

                // Store reference for potential API access
                container.webllmInterface = chatInterface;
            });

            // Initialize simple chat blocks
            $(context).find('[data-webllm-chat]').once('webllm-simple').each(function () {
                const $element = $(this);
                const modelId = $element.data('webllm-model');
                const autoLoad = $element.data('webllm-autoload');

                if (modelId) {
                    // Create a simple interface for predefined model
                    const engineManager = new Drupal.webllmAI.EngineManager();

                    if (autoLoad) {
                        // Auto-load the model
                        engineManager.initializeEngine(modelId, (progress) => {
                            $element.find('.webllm-progress').text(
                                progress.text || `Loading... ${Math.round(progress.progress * 100)}%`
                            );
                        }).then(() => {
                            $element.find('.webllm-status').text('Ready');
                            $element.trigger('webllm:ready', [engineManager]);
                        }).catch(error => {
                            $element.find('.webllm-status').text('Error: ' + error.message);
                        });
                    }

                    // Store engine manager reference
                    this.webllmEngine = engineManager;
                }
            });
        }
    };

    /**
     * API for other modules to interact with WebLLM
     */
    Drupal.webllmAI.api = {
        /**
         * Create a new engine instance
         */
        createEngine: function () {
            return new Drupal.webllmAI.EngineManager();
        },

        /**
         * Get engine from a container element
         */
        getEngine: function (containerId) {
            const container = document.getElementById(containerId);
            return container ? container.webllmEngine : null;
        },

        /**
         * Quick chat completion (for programmatic use)
         */
        quickCompletion: async function (prompt, modelId = null, options = {}) {
            const engine = new Drupal.webllmAI.EngineManager();

            if (!modelId) {
                modelId = 'Llama-3.1-8B-Instruct-q4f32_1-MLC'; // Default model
            }

            await engine.initializeEngine(modelId);

            const messages = [
                { role: 'user', content: prompt }
            ];

            const response = await engine.generateCompletion(messages, options);
            return response.choices[0].message.content;
        }
    };

})(Drupal, drupalSettings);
