<?php

namespace Drupal\webllm_ai\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\webllm_ai\Service\WebLlmService;
use Drupal\webllm_ai\Service\ModelManager;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;

/**
 * Controller for RAG (Retrieval-Augmented Generation) Demo.
 */
class RagDemoController extends ControllerBase {

  /**
   * The Web-LLM service.
   *
   * @var \Drupal\webllm_ai\Service\WebLlmService
   */
  protected $webllmService;

  /**
   * The model manager service.
   *
   * @var \Drupal\webllm_ai\Service\ModelManager
   */
  protected $modelManager;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Constructs a new RagDemoController object.
   *
   * @param \Drupal\webllm_ai\Service\WebLlmService $webllm_service
   *   The Web-LLM service.
   * @param \Drupal\webllm_ai\Service\ModelManager $model_manager
   *   The model manager service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   */
  public function __construct(WebLlmService $webllm_service, ModelManager $model_manager, ConfigFactoryInterface $config_factory) {
    $this->webllmService = $webllm_service;
    $this->modelManager = $model_manager;
    $this->configFactory = $config_factory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('webllm_ai.service'),
      $container->get('webllm_ai.model_manager'),
      $container->get('config.factory')
    );
  }

  /**
   * RAG Demo page.
   *
   * @return array
   *   A render array.
   */
  public function ragDemo() {
    $config = $this->configFactory->get('webllm_ai.settings');
    
    if (!$config->get('enabled')) {
      $this->messenger()->addWarning($this->t('Web-LLM AI module is currently disabled. Please enable it in the settings.'));
    }

    $build = [
      '#theme' => 'webllm_ai_rag_demo',
      '#attached' => [
        'library' => ['webllm_ai/rag-demo'],
        'drupalSettings' => [
          'webllm_ai' => [
            'enabled' => $config->get('enabled'),
            'default_model' => $config->get('default_model'),
            'max_tokens' => $config->get('max_tokens'),
            'temperature' => $config->get('temperature'),
            'debug_mode' => $config->get('debug_mode'),
            'api_endpoints' => [
              'models' => '/webllm-ai/api/models',
              'config' => '/webllm-ai/api/config',
              'health' => '/webllm-ai/api/health',
              'progress' => '/webllm-ai/api/model-progress',
              'extract_content' => '/webllm-ai/api/extract-content',
              'process_rag' => '/webllm-ai/api/process-rag',
            ],
          ],
        ],
      ],
      '#cache' => [
        'contexts' => ['user.permissions'],
        'tags' => ['config:webllm_ai.settings'],
        'max-age' => 3600,
      ],
    ];

    return $build;
  }
  /**
   * Extract content from the current page.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request object.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   JSON response with extracted content.
   */
  public function extractContent(Request $request) {
    $data = json_decode($request->getContent(), TRUE);
    $page_content = $data['page_content'] ?? '';
    $page_url = $data['page_url'] ?? '';
    $page_title = $data['page_title'] ?? '';

    if (empty($page_content)) {
      return new JsonResponse(['error' => 'Page content is required'], 400);
    }

    try {
      // Extract text content from the provided HTML
      $content = $this->extractTextFromHtml($page_content);
      
      if (empty($content)) {
        return new JsonResponse(['error' => 'No readable content found on this page'], 400);
      }

      // Chunk the content for better processing
      $chunks = $this->chunkContent($content, 1000);

      return new JsonResponse([
        'success' => true,
        'url' => $page_url,
        'title' => $page_title,
        'content' => $content,
        'chunks' => $chunks,
        'word_count' => str_word_count($content),
        'char_count' => strlen($content),
      ]);

    } catch (\Exception $e) {
      return new JsonResponse(['error' => 'Failed to process content: ' . $e->getMessage()], 500);
    }
  }

  /**
   * Process RAG query with context.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request object.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   JSON response with processing status.
   */
  public function processRag(Request $request) {
    $data = json_decode($request->getContent(), TRUE);
    $query = $data['query'] ?? '';
    $context = $data['context'] ?? '';

    if (empty($query)) {
      return new JsonResponse(['error' => 'Query is required'], 400);
    }

    if (empty($context)) {
      return new JsonResponse(['error' => 'Context is required'], 400);
    }

    try {
      // Create RAG prompt
      $prompt = $this->createRagPrompt($query, $context);
      
      return new JsonResponse([
        'success' => true,
        'prompt' => $prompt,
        'query' => $query,
        'context_length' => strlen($context),
      ]);

    } catch (\Exception $e) {
      return new JsonResponse(['error' => 'Failed to process RAG query: ' . $e->getMessage()], 500);
    }
  }

  /**
   * Extract readable text from HTML.
   *
   * @param string $html
   *   The HTML content.
   *
   * @return string
   *   Extracted text content.
   */
  private function extractTextFromHtml($html) {
    // Remove script and style elements
    $html = preg_replace('/<script\b[^<]*(?:(?!<\/script>)<[^<]*)*<\/script>/mi', '', $html);
    $html = preg_replace('/<style\b[^<]*(?:(?!<\/style>)<[^<]*)*<\/style>/mi', '', $html);
    
    // Remove HTML tags
    $text = strip_tags($html);
    
    // Clean up whitespace
    $text = preg_replace('/\s+/', ' ', $text);
    $text = trim($text);
    
    return $text;
  }

  /**
   * Chunk content into smaller pieces.
   *
   * @param string $content
   *   The content to chunk.
   * @param int $maxLength
   *   Maximum length per chunk.
   *
   * @return array
   *   Array of content chunks.
   */
  private function chunkContent($content, $maxLength = 1000) {
    $chunks = [];
    $sentences = preg_split('/(?<=[.!?])\s+/', $content);
    
    $currentChunk = '';
    
    foreach ($sentences as $sentence) {
      if (strlen($currentChunk . ' ' . $sentence) > $maxLength && !empty($currentChunk)) {
        $chunks[] = trim($currentChunk);
        $currentChunk = $sentence;
      } else {
        $currentChunk .= (empty($currentChunk) ? '' : ' ') . $sentence;
      }
    }
    
    if (!empty($currentChunk)) {
      $chunks[] = trim($currentChunk);
    }
    
    return $chunks;
  }

  /**
   * Create RAG prompt with context and query.
   *
   * @param string $query
   *   The user query.
   * @param string $context
   *   The context from the website.
   *
   * @return string
   *   The formatted RAG prompt.
   */
  private function createRagPrompt($query, $context) {
    $prompt = "You are an intelligent assistant that answers questions based on the provided context from a website.\n\n";
    $prompt .= "CONTEXT:\n";
    $prompt .= "=======\n";
    $prompt .= substr($context, 0, 3000) . (strlen($context) > 3000 ? '...' : '') . "\n\n";
    $prompt .= "INSTRUCTIONS:\n";
    $prompt .= "============\n";
    $prompt .= "1. Answer the question based ONLY on the information provided in the context above\n";
    $prompt .= "2. If the context doesn't contain enough information to answer the question, say so\n";
    $prompt .= "3. Be accurate and concise in your response\n";
    $prompt .= "4. Cite specific parts of the context when relevant\n\n";
    $prompt .= "QUESTION: " . $query . "\n\n";
    $prompt .= "ANSWER:";

    return $prompt;
  }
}
