/**
 * @file
 * RAG Demo functionality for WebLLM AI.
 */

(function ($, Drupal, drupalSettings) {
    'use strict';

    let webllmEngine = null;
    let extractedContent = '';
    let availableModels = [];
    let currentModel = null;

    Drupal.behaviors.webllmRagDemo = {
        attach: function (context, settings) {
            if (!settings.webllm_ai || !settings.webllm_ai.enabled) {
                showStatus('WebLLM AI is not enabled.', 'error');
                return;
            }

            // Initialize the demo
            initializeRagDemo();
        }
    };
    /**
     * Initialize the RAG demo.
     */
    function initializeRagDemo() {
        // Bind event handlers
        $('#extract-content-btn').once('rag-extract').on('click', extractCurrentPageContent);
        $('#load-model-btn').once('rag-load-model').on('click', loadSelectedModel);
        $('#ask-question-btn').once('rag-ask').on('click', askQuestion);

        // Suggested question clicks
        $(document).on('click', '.suggested-question', function () {
            $('#question-input').val($(this).text());
        });

        // Enter key handling for question input
        $('#question-input').on('keypress', function (e) {
            if (e.which === 13) {
                askQuestion();
            }
        });

        // Load available models
        loadAvailableModels();

        // Check WebGPU support
        checkWebGpuSupport();

        // Initialize current page info
        initializeCurrentPageInfo();
    }

    /**
     * Initialize current page information.
     */
    function initializeCurrentPageInfo() {
        $('#current-page-title').text(document.title || 'Untitled Page');
        $('#current-page-url').text(window.location.href);
    }

    /**
     * Extract content from the current page.
     */
    function extractCurrentPageContent() {
        showLoading('Extracting content from current page...');

        // Get the main content from the current page
        const pageContent = extractPageContent();

        if (!pageContent.trim()) {
            hideLoading();
            showStatus('No readable content found on this page.', 'error');
            return;
        }

        // Send the content to the backend for processing
        $.ajax({
            url: drupalSettings.webllm_ai.api_endpoints.extract_content,
            method: 'POST',
            contentType: 'application/json',
            data: JSON.stringify({
                page_content: pageContent,
                page_url: window.location.href,
                page_title: document.title
            }),
            success: function (response) {
                hideLoading();

                if (response.success) {
                    extractedContent = response.content;
                    displayExtractedContent(response);
                    showContentSection();
                    showStatus('Page content extracted successfully!', 'success');
                } else {
                    showStatus('Error: ' + response.error, 'error');
                }
            },
            error: function (xhr) {
                hideLoading();
                const error = xhr.responseJSON ? xhr.responseJSON.error : 'Failed to extract content';
                showStatus('Error: ' + error, 'error');
            }
        });
    }

    /**
     * Extract readable content from the current page.
     */
    function extractPageContent() {
        // Try to get main content areas first
        let content = '';

        // Common content selectors (in order of preference)
        const contentSelectors = [
            'main',
            '[role="main"]',
            '.main-content',
            '.content',
            '.post-content',
            '.entry-content',
            '.article-content',
            '#content',
            '#main-content',
            '.page-content',
            'article',
            '.node-content'
        ];

        // Try each selector until we find content
        for (const selector of contentSelectors) {
            const element = document.querySelector(selector);
            if (element) {
                content = element.innerText || element.textContent || '';
                if (content.trim().length > 100) { // Ensure we have substantial content
                    break;
                }
            }
        }

        // Fallback to body content if no main content found
        if (!content.trim() || content.trim().length < 100) {
            const bodyElement = document.body.cloneNode(true);

            // Remove script, style, nav, footer, and other non-content elements
            const removeSelectors = [
                'script', 'style', 'nav', 'footer', 'header',
                '.navigation', '.nav', '.menu', '.sidebar',
                '.widget', '.advertisement', '.ads', '.social',
                '.comments', '.comment-form', '.breadcrumb'
            ];

            removeSelectors.forEach(selector => {
                const elements = bodyElement.querySelectorAll(selector);
                elements.forEach(el => el.remove());
            });

            content = bodyElement.innerText || bodyElement.textContent || '';
        }

        // Clean up the content
        content = content
            .replace(/\s+/g, ' ') // Replace multiple whitespace with single space
            .replace(/\n+/g, '\n') // Replace multiple newlines with single newline
            .trim();

        return content;
    }
    /**
     * Display the extracted content.
     */
    function displayExtractedContent(data) {
        $('#extracted-content').val(data.content);
        $('#word-count').text(data.word_count);
        $('#char-count').text(data.char_count);
        $('#source-url').text(data.url);

        // Show model selection section
        $('.model-selection-section').show();
    }

    /**
     * Show the content display section.
     */
    function showContentSection() {
        $('.content-display-section').show();
    }

    /**
     * Load available models.
     */
    function loadAvailableModels() {
        $.ajax({
            url: drupalSettings.webllm_ai.api_endpoints.models,
            method: 'GET',
            success: function (response) {
                if (response.models && response.models.length > 0) {
                    availableModels = response.models;
                    populateModelSelect();
                } else {
                    showStatus('No models available.', 'warning');
                }
            },
            error: function () {
                showStatus('Failed to load available models.', 'error');
            }
        });
    }

    /**
     * Populate the model selection dropdown.
     */
    function populateModelSelect() {
        const select = $('#model-select');
        select.empty();
        select.append('<option value="">Select a model...</option>');

        availableModels.forEach(function (model) {
            select.append(`<option value="${model.model_id}">${model.model_id}</option>`);
        });

        // Set default model if available
        const defaultModel = drupalSettings.webllm_ai.default_model;
        if (defaultModel) {
            select.val(defaultModel);
        }
    }

    /**
     * Load the selected model.
     */
    function loadSelectedModel() {
        const selectedModel = $('#model-select').val();

        if (!selectedModel) {
            showStatus('Please select a model first.', 'error');
            return;
        }

        if (currentModel === selectedModel && webllmEngine) {
            showChatSection();
            showStatus('Model already loaded!', 'success');
            return;
        }

        initializeWebLLM(selectedModel);
    }

    /**
     * Initialize WebLLM with the selected model.
     */
    async function initializeWebLLM(modelId) {
        try {
            showModelLoading();

            // Import WebLLM dynamically
            const { CreateMLCEngine } = await import('https://esm.run/@mlc-ai/web-llm');

            // Create engine with progress callback
            webllmEngine = await CreateMLCEngine(modelId, {
                initProgressCallback: updateProgress
            });

            currentModel = modelId;
            hideModelLoading();
            showChatSection();
            showStatus('Model loaded successfully!', 'success');

        } catch (error) {
            hideModelLoading();
            showStatus('Failed to load model: ' + error.message, 'error');
            console.error('WebLLM initialization error:', error);
        }
    }

    /**
     * Update loading progress.
     */
    function updateProgress(progress) {
        const percentage = Math.round(progress.progress * 100);
        $('#progress-bar').css('width', percentage + '%');
        $('#progress-text').text(percentage + '%');
        $('#loading-status').text(progress.text || 'Loading...');
    }

    /**
     * Show model loading interface.
     */
    function showModelLoading() {
        $('#loading-progress').show();
        $('#load-model-btn').prop('disabled', true).text('Loading...');
    }

    /**
     * Hide model loading interface.
     */
    function hideModelLoading() {
        $('#loading-progress').hide();
        $('#load-model-btn').prop('disabled', false).text('Load Model');
    }

    /**
     * Show the chat section.
     */
    function showChatSection() {
        $('.chat-section').show();
        $('#question-input').focus();
    }

    /**
     * Ask a question about the content.
     */
    async function askQuestion() {
        const question = $('#question-input').val().trim();

        if (!question) {
            showStatus('Please enter a question.', 'error');
            return;
        }

        if (!webllmEngine) {
            showStatus('Please load a model first.', 'error');
            return;
        } if (!extractedContent) {
            showStatus('Please extract page content first.', 'error');
            return;
        }

        // Add user message to chat
        addMessageToChat('user', question);

        // Clear input
        $('#question-input').val('');

        // Show thinking indicator
        const thinkingId = addMessageToChat('assistant', 'Thinking...');

        try {
            // Create RAG prompt
            const ragPromptResponse = await $.ajax({
                url: drupalSettings.webllm_ai.api_endpoints.process_rag,
                method: 'POST',
                contentType: 'application/json',
                data: JSON.stringify({
                    query: question,
                    context: extractedContent
                })
            });

            if (!ragPromptResponse.success) {
                throw new Error(ragPromptResponse.error);
            }

            // Generate response using WebLLM
            const response = await webllmEngine.chat.completions.create({
                messages: [{ role: 'user', content: ragPromptResponse.prompt }],
                temperature: parseFloat(drupalSettings.webllm_ai.temperature) || 0.7,
                max_tokens: parseInt(drupalSettings.webllm_ai.max_tokens) || 1000,
            });

            // Update the assistant message
            const assistantMessage = response.choices[0].message.content;
            updateChatMessage(thinkingId, assistantMessage);

        } catch (error) {
            updateChatMessage(thinkingId, 'Sorry, I encountered an error: ' + error.message);
            showStatus('Error generating response: ' + error.message, 'error');
        }
    }

    /**
     * Add a message to the chat.
     */
    function addMessageToChat(role, content) {
        const messageId = 'msg-' + Date.now() + '-' + Math.random().toString(36).substr(2, 9);
        const messageClass = role === 'user' ? 'user-message' : 'assistant-message';

        const messageHtml = `
      <div class="chat-message ${messageClass}" id="${messageId}">
        <div class="message-role">${role === 'user' ? 'You' : 'AI Assistant'}</div>
        <div class="message-content">${escapeHtml(content)}</div>
        <div class="message-time">${new Date().toLocaleTimeString()}</div>
      </div>
    `;

        $('#chat-messages').append(messageHtml);
        $('#chat-messages').scrollTop($('#chat-messages')[0].scrollHeight);

        return messageId;
    }

    /**
     * Update a chat message.
     */
    function updateChatMessage(messageId, content) {
        $(`#${messageId} .message-content`).html(escapeHtml(content));
        $('#chat-messages').scrollTop($('#chat-messages')[0].scrollHeight);
    }

    /**
     * Check WebGPU support.
     */
    async function checkWebGpuSupport() {
        if (!navigator.gpu) {
            showStatus('WebGPU is not supported in this browser. The AI will run slower on CPU.', 'warning');
            return;
        }

        try {
            const adapter = await navigator.gpu.requestAdapter();
            if (adapter) {
                showStatus('WebGPU is supported! AI will run with GPU acceleration.', 'info');
            } else {
                showStatus('WebGPU adapter not available. The AI will run on CPU.', 'warning');
            }
        } catch (error) {
            showStatus('WebGPU check failed. The AI will run on CPU.', 'warning');
        }
    }

    /**
     * Show status message.
     */
    function showStatus(message, type) {
        const alertClass = {
            'success': 'alert-success',
            'error': 'alert-danger',
            'warning': 'alert-warning',
            'info': 'alert-info'
        }[type] || 'alert-info';

        const statusHtml = `
      <div class="alert ${alertClass} alert-dismissible fade show" role="alert">
        ${escapeHtml(message)}
        <button type="button" class="btn-close" data-bs-dismiss="alert"></button>
      </div>
    `;

        $('#status-messages').append(statusHtml);

        // Auto-remove after 5 seconds for non-error messages
        if (type !== 'error') {
            setTimeout(() => {
                $('#status-messages .alert').first().fadeOut(() => {
                    $(this).remove();
                });
            }, 5000);
        }
    }

    /**
     * Show loading overlay.
     */
    function showLoading(message) {
        $('#loading-overlay .loading-text').text(message);
        $('#loading-overlay').show();
    }

    /**
     * Hide loading overlay.
     */
    function hideLoading() {
        $('#loading-overlay').hide();
    }

    /**
     * Escape HTML to prevent XSS.
     */
    function escapeHtml(text) {
        const div = document.createElement('div');
        div.textContent = text;
        return div.innerHTML;
    }

})(jQuery, Drupal, drupalSettings);
