<?php

namespace Drupal\webllm_ai\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a 'Web-LLM AI Chat' Block.
 *
 * @Block(
 *   id = "webllm_ai_chat_block",
 *   admin_label = @Translation("Web-LLM AI Chat"),
 *   category = @Translation("Web-LLM AI"),
 * )
 */
class WebLLMChatBlock extends BlockBase implements ContainerFactoryPluginInterface {

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Constructs a new WebLLMChatBlock instance.
   *
   * @param array $configuration
   *   The plugin configuration.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, ConfigFactoryInterface $config_factory) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->configFactory = $config_factory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('config.factory')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'model' => '',
      'height' => '400px',
      'show_model_selector' => TRUE,
      'show_performance_stats' => FALSE,
      'placeholder_text' => 'Ask me anything...',
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $form = parent::blockForm($form, $form_state);
    $config = $this->getConfiguration();

    $form['model'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Default Model'),
      '#description' => $this->t('Specify a default model ID (e.g., Llama-3.2-3B-Instruct-q4f32_1-MLC). Leave empty to use system default.'),
      '#default_value' => $config['model'],
    ];

    $form['height'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Block Height'),
      '#description' => $this->t('Specify the height of the chat interface (e.g., 400px, 50vh).'),
      '#default_value' => $config['height'],
      '#required' => TRUE,
    ];

    $form['show_model_selector'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Show Model Selector'),
      '#description' => $this->t('Allow users to choose different models.'),
      '#default_value' => $config['show_model_selector'],
    ];

    $form['show_performance_stats'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Show Performance Statistics'),
      '#description' => $this->t('Display tokens per second and other performance metrics.'),
      '#default_value' => $config['show_performance_stats'],
    ];

    $form['placeholder_text'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Placeholder Text'),
      '#description' => $this->t('Text shown in the input field when empty.'),
      '#default_value' => $config['placeholder_text'],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    parent::blockSubmit($form, $form_state);
    $values = $form_state->getValues();
    $this->configuration['model'] = $values['model'];
    $this->configuration['height'] = $values['height'];
    $this->configuration['show_model_selector'] = $values['show_model_selector'];
    $this->configuration['show_performance_stats'] = $values['show_performance_stats'];
    $this->configuration['placeholder_text'] = $values['placeholder_text'];
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $config = $this->getConfiguration();
    $module_config = $this->configFactory->get('webllm_ai.settings');

    // Check if module is enabled
    if (!$module_config->get('enabled')) {
      return [
        '#markup' => '<div class="webllm-ai-disabled">' . $this->t('Web-LLM AI is currently disabled.') . '</div>',
      ];
    }

    $build = [
      '#theme' => 'webllm_ai_chat_block',
      '#configuration' => $config,
      '#attached' => [
        'library' => ['webllm_ai/webllm-chat'],
        'drupalSettings' => [
          'webllm_ai' => [
            'block_config' => [
              'model' => $config['model'] ?: $module_config->get('default_model'),
              'height' => $config['height'],
              'show_model_selector' => $config['show_model_selector'],
              'show_performance_stats' => $config['show_performance_stats'],
              'placeholder_text' => $config['placeholder_text'],
            ],
            'enabled' => $module_config->get('enabled'),
            'max_tokens' => $module_config->get('max_tokens'),
            'temperature' => $module_config->get('temperature'),
            'debug_mode' => $module_config->get('debug_mode'),
            'api_endpoints' => [
              'models' => '/webllm-ai/api/models',
              'config' => '/webllm-ai/api/config',
              'health' => '/webllm-ai/api/health',
            ],
          ],
        ],
      ],
      '#cache' => [
        'contexts' => ['user.permissions'],
        'tags' => ['config:webllm_ai.settings'],
        'max-age' => 3600,
      ],
    ];

    return $build;
  }

}
