<?php

namespace Drupal\webllm_ai\Service;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Cache\CacheBackendInterface;

/**
 * Model manager for WebLLM AI models.
 */
class ModelManager {

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The cache backend.
   *
   * @var \Drupal\Core\Cache\CacheBackendInterface
   */
  protected $cache;

  /**
   * Constructs a ModelManager object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\Core\Cache\CacheBackendInterface $cache
   *   The cache backend.
   */
  public function __construct(ConfigFactoryInterface $config_factory, CacheBackendInterface $cache) {
    $this->configFactory = $config_factory;
    $this->cache = $cache;
  }

  /**
   * Get all available model definitions.
   *
   * @return array
   *   Array of model definitions.
   */
  public function getAllModels() {
    $cache_key = 'webllm_ai_all_models';
    
    if ($cached = $this->cache->get($cache_key)) {
      return $cached->data;
    }

    $models = [
      'Llama-3.1-8B-Instruct-q4f32_1-MLC' => [
        'id' => 'Llama-3.1-8B-Instruct-q4f32_1-MLC',
        'name' => 'Llama 3.1 8B Instruct',
        'family' => 'llama',
        'version' => '3.1',
        'size' => '8B',
        'quantization' => 'q4f32_1',
        'description' => 'Meta\'s Llama 3.1 8B model optimized for instruction following',
        'capabilities' => ['chat', 'completion', 'instruction-following'],
        'languages' => ['en', 'es', 'fr', 'de', 'it', 'pt', 'hi', 'th'],
        'context_length' => 131072,
        'recommended_memory' => 6144, // MB
        'download_size' => 4800, // MB (approximate)
        'performance_tier' => 'medium',
        'license' => 'Custom (Llama 3.1)',
        'tags' => ['recommended', 'general-purpose', 'efficient'],
      ],
      'Llama-3.1-70B-Instruct-q4f16_1-MLC' => [
        'id' => 'Llama-3.1-70B-Instruct-q4f16_1-MLC',
        'name' => 'Llama 3.1 70B Instruct',
        'family' => 'llama',
        'version' => '3.1',
        'size' => '70B',
        'quantization' => 'q4f16_1',
        'description' => 'Meta\'s largest Llama 3.1 model with exceptional quality',
        'capabilities' => ['chat', 'completion', 'instruction-following', 'reasoning'],
        'languages' => ['en', 'es', 'fr', 'de', 'it', 'pt', 'hi', 'th'],
        'context_length' => 131072,
        'recommended_memory' => 49152, // MB
        'download_size' => 40960, // MB (approximate)
        'performance_tier' => 'high',
        'license' => 'Custom (Llama 3.1)',
        'tags' => ['high-quality', 'large', 'resource-intensive'],
      ],
      'Phi-3.5-mini-instruct-q4f16_1-MLC' => [
        'id' => 'Phi-3.5-mini-instruct-q4f16_1-MLC',
        'name' => 'Phi 3.5 Mini Instruct',
        'family' => 'phi',
        'version' => '3.5',
        'size' => '3.8B',
        'quantization' => 'q4f16_1',
        'description' => 'Microsoft\'s compact and efficient Phi model',
        'capabilities' => ['chat', 'completion', 'instruction-following'],
        'languages' => ['en'],
        'context_length' => 131072,
        'recommended_memory' => 3072, // MB
        'download_size' => 2300, // MB (approximate)
        'performance_tier' => 'fast',
        'license' => 'MIT',
        'tags' => ['fast', 'lightweight', 'mobile-friendly'],
      ],
      'Qwen2.5-7B-Instruct-q4f16_1-MLC' => [
        'id' => 'Qwen2.5-7B-Instruct-q4f16_1-MLC',
        'name' => 'Qwen 2.5 7B Instruct',
        'family' => 'qwen',
        'version' => '2.5',
        'size' => '7B',
        'quantization' => 'q4f16_1',
        'description' => 'Alibaba\'s multilingual Qwen model',
        'capabilities' => ['chat', 'completion', 'instruction-following', 'multilingual'],
        'languages' => ['en', 'zh', 'ja', 'ko', 'ar', 'es', 'fr', 'ru'],
        'context_length' => 131072,
        'recommended_memory' => 5120, // MB
        'download_size' => 4200, // MB (approximate)
        'performance_tier' => 'medium',
        'license' => 'Apache 2.0',
        'tags' => ['multilingual', 'chinese', 'versatile'],
      ],
      'Qwen2.5-14B-Instruct-q4f16_1-MLC' => [
        'id' => 'Qwen2.5-14B-Instruct-q4f16_1-MLC',
        'name' => 'Qwen 2.5 14B Instruct',
        'family' => 'qwen',
        'version' => '2.5',
        'size' => '14B',
        'quantization' => 'q4f16_1',
        'description' => 'Larger Qwen model with enhanced capabilities',
        'capabilities' => ['chat', 'completion', 'instruction-following', 'multilingual', 'reasoning'],
        'languages' => ['en', 'zh', 'ja', 'ko', 'ar', 'es', 'fr', 'ru'],
        'context_length' => 131072,
        'recommended_memory' => 10240, // MB
        'download_size' => 8400, // MB (approximate)
        'performance_tier' => 'medium-high',
        'license' => 'Apache 2.0',
        'tags' => ['multilingual', 'chinese', 'enhanced'],
      ],
      'Gemma-2-9B-It-q4f16_1-MLC' => [
        'id' => 'Gemma-2-9B-It-q4f16_1-MLC',
        'name' => 'Gemma 2 9B IT',
        'family' => 'gemma',
        'version' => '2',
        'size' => '9B',
        'quantization' => 'q4f16_1',
        'description' => 'Google\'s Gemma 2 model for instruction tasks',
        'capabilities' => ['chat', 'completion', 'instruction-following'],
        'languages' => ['en'],
        'context_length' => 8192,
        'recommended_memory' => 6144, // MB
        'download_size' => 5100, // MB (approximate)
        'performance_tier' => 'medium',
        'license' => 'Gemma Terms of Use',
        'tags' => ['google', 'efficient', 'research'],
      ],
      'Mistral-7B-Instruct-v0.3-q4f16_1-MLC' => [
        'id' => 'Mistral-7B-Instruct-v0.3-q4f16_1-MLC',
        'name' => 'Mistral 7B Instruct v0.3',
        'family' => 'mistral',
        'version' => '0.3',
        'size' => '7B',
        'quantization' => 'q4f16_1',
        'description' => 'Mistral AI\'s instruction-tuned model',
        'capabilities' => ['chat', 'completion', 'instruction-following', 'function-calling'],
        'languages' => ['en', 'fr', 'es', 'de', 'it'],
        'context_length' => 32768,
        'recommended_memory' => 5120, // MB
        'download_size' => 4200, // MB (approximate)
        'performance_tier' => 'medium',
        'license' => 'Apache 2.0',
        'tags' => ['efficient', 'function-calling', 'european'],
      ],
    ];

    // Cache for 1 hour
    $this->cache->set($cache_key, $models, time() + 3600);
    
    return $models;
  }

  /**
   * Get model by ID.
   *
   * @param string $model_id
   *   The model ID.
   *
   * @return array|null
   *   The model definition or NULL if not found.
   */
  public function getModel($model_id) {
    $models = $this->getAllModels();
    return isset($models[$model_id]) ? $models[$model_id] : NULL;
  }

  /**
   * Get models by family.
   *
   * @param string $family
   *   The model family (e.g., 'llama', 'phi').
   *
   * @return array
   *   Array of models in the family.
   */
  public function getModelsByFamily($family) {
    $models = $this->getAllModels();
    return array_filter($models, function($model) use ($family) {
      return $model['family'] === $family;
    });
  }

  /**
   * Get models by performance tier.
   *
   * @param string $tier
   *   The performance tier (fast, medium, medium-high, high).
   *
   * @return array
   *   Array of models in the tier.
   */
  public function getModelsByTier($tier) {
    $models = $this->getAllModels();
    return array_filter($models, function($model) use ($tier) {
      return $model['performance_tier'] === $tier;
    });
  }

  /**
   * Get models by memory requirement.
   *
   * @param int $max_memory_mb
   *   Maximum memory in MB.
   *
   * @return array
   *   Array of models that fit within memory constraint.
   */
  public function getModelsByMemory($max_memory_mb) {
    $models = $this->getAllModels();
    return array_filter($models, function($model) use ($max_memory_mb) {
      return $model['recommended_memory'] <= $max_memory_mb;
    });
  }

  /**
   * Get models by language support.
   *
   * @param string $language
   *   Language code (e.g., 'en', 'zh').
   *
   * @return array
   *   Array of models that support the language.
   */
  public function getModelsByLanguage($language) {
    $models = $this->getAllModels();
    return array_filter($models, function($model) use ($language) {
      return in_array($language, $model['languages']);
    });
  }

  /**
   * Get recommended models based on criteria.
   *
   * @param array $criteria
   *   Criteria for recommendation.
   *
   * @return array
   *   Array of recommended models.
   */
  public function getRecommendedModels(array $criteria = []) {
    $models = $this->getAllModels();
    
    // Default criteria
    $default_criteria = [
      'max_memory_mb' => 8192,
      'performance_preference' => 'balanced',
      'language' => 'en',
    ];
    
    $criteria = array_merge($default_criteria, $criteria);
    
    // Filter by memory if specified
    if (isset($criteria['max_memory_mb'])) {
      $models = array_filter($models, function($model) use ($criteria) {
        return $model['recommended_memory'] <= $criteria['max_memory_mb'];
      });
    }
    
    // Filter by language if specified
    if (isset($criteria['language'])) {
      $models = array_filter($models, function($model) use ($criteria) {
        return in_array($criteria['language'], $model['languages']);
      });
    }
    
    // Sort by performance preference
    if (isset($criteria['performance_preference'])) {
      $preference = $criteria['performance_preference'];
      
      switch ($preference) {
        case 'speed':
          // Prefer fast models
          uasort($models, function($a, $b) {
            $tiers = ['fast' => 4, 'medium' => 3, 'medium-high' => 2, 'high' => 1];
            return $tiers[$b['performance_tier']] <=> $tiers[$a['performance_tier']];
          });
          break;
          
        case 'quality':
          // Prefer high-quality models
          uasort($models, function($a, $b) {
            $tiers = ['high' => 4, 'medium-high' => 3, 'medium' => 2, 'fast' => 1];
            return $tiers[$b['performance_tier']] <=> $tiers[$a['performance_tier']];
          });
          break;
          
        case 'balanced':
        default:
          // Prefer medium tier models
          uasort($models, function($a, $b) {
            $tiers = ['medium' => 4, 'medium-high' => 3, 'fast' => 2, 'high' => 1];
            return $tiers[$b['performance_tier']] <=> $tiers[$a['performance_tier']];
          });
          break;
      }
    }
    
    return $models;
  }

  /**
   * Get model selection advice based on user context.
   *
   * @param array $context
   *   User context information.
   *
   * @return array
   *   Model selection advice.
   */
  public function getModelAdvice(array $context = []) {
    $advice = [
      'recommended_model' => 'Llama-3.1-8B-Instruct-q4f32_1-MLC',
      'alternatives' => [],
      'warnings' => [],
      'tips' => [],
    ];

    // Analyze available memory
    if (isset($context['available_memory_gb'])) {
      $memory_gb = $context['available_memory_gb'];
      
      if ($memory_gb < 4) {
        $advice['recommended_model'] = 'Phi-3.5-mini-instruct-q4f16_1-MLC';
        $advice['warnings'][] = 'Limited memory detected. Using lightweight model.';
      } elseif ($memory_gb >= 32) {
        $advice['recommended_model'] = 'Llama-3.1-70B-Instruct-q4f16_1-MLC';
        $advice['tips'][] = 'High memory available. Consider using the 70B model for best quality.';
      }
    }

    // Analyze use case
    if (isset($context['use_case'])) {
      switch ($context['use_case']) {
        case 'multilingual':
          $advice['recommended_model'] = 'Qwen2.5-7B-Instruct-q4f16_1-MLC';
          $advice['tips'][] = 'Qwen models excel at multilingual tasks.';
          break;
          
        case 'speed':
          $advice['recommended_model'] = 'Phi-3.5-mini-instruct-q4f16_1-MLC';
          $advice['tips'][] = 'Phi-3.5 Mini provides fastest response times.';
          break;
          
        case 'function_calling':
          $advice['recommended_model'] = 'Mistral-7B-Instruct-v0.3-q4f16_1-MLC';
          $advice['tips'][] = 'Mistral models support function calling capabilities.';
          break;
      }
    }

    return $advice;
  }

}
