/**
 * @file
 * Simplified RAG Demo functionality for WebLLM AI.
 * Uses the main webllm_ai.js engine and defaults to Phi-3.5-mini for testing.
 */

(function ($, Drupal, drupalSettings) {
  'use strict';

  let engineManager = null;
  let extractedContent = '';
  const DEFAULT_MODEL = 'Phi-3.5-mini-instruct-q4f16_1-MLC'; // Lightweight testing model

  Drupal.behaviors.webllmRagDemo = {
    attach: function (context, settings) {
      console.log('🚀 RAG Demo initializing...');
      
      // Check if settings are available
      if (!settings.webllm_ai) {
        console.error('❌ WebLLM AI settings not found');
        showStatus('WebLLM AI module settings not found. Please check module configuration.', 'error');
        return;
      }
      
      console.log('✅ WebLLM AI settings:', settings.webllm_ai);
      
      // Initialize WebLLM Engine Manager from main module
      if (Drupal.webllmAI && Drupal.webllmAI.EngineManager) {
        engineManager = new Drupal.webllmAI.EngineManager();
        console.log('✅ WebLLM Engine Manager initialized');
      } else {
        console.error('❌ WebLLM AI main module not loaded');
        showStatus('WebLLM AI main module not loaded. Please check dependencies.', 'error');
        return;
      }
      
      // Initialize page info
      $('#current-page-title').text(document.title || 'Untitled Page');
      $('#current-page-url').text(window.location.href);
      
      // Load available models and set default
      loadAvailableModels();
      
      // Bind extract content button
      $('#extract-content-btn').once('rag-extract').on('click', function() {
        console.log('🎯 Extract button clicked');
        extractContent();
      });
      
      // Bind load model button
      $('#load-model-btn').once('rag-load-model').on('click', function() {
        console.log('🎯 Load model button clicked');
        loadSelectedModel();
      });
      
      // Bind ask question button
      $('#ask-question-btn').once('rag-ask').on('click', function() {
        console.log('🎯 Ask question button clicked');
        askQuestion();
      });
      
      // Enter key handling for question input
      $('#question-input').on('keypress', function (e) {
        if (e.which === 13) {
          askQuestion();
        }
      });
      
      // Suggested question clicks
      $(document).on('click', '.suggested-question', function () {
        $('#question-input').val($(this).text());
      });
      
      // Debug mode toggle
      $('#debug-mode-toggle').once('rag-debug').on('change', function() {
        const debugMode = $(this).is(':checked');
        $('#debug-info').toggle(debugMode);
        console.log('🐛 Debug mode:', debugMode ? 'ON' : 'OFF');
      });
      
      console.log('✅ RAG Demo ready');
    }
  };

  async function loadAvailableModels() {
    try {
      console.log('📋 Loading available models...');
      const models = await engineManager.getAvailableModels();
      
      const select = $('#model-select');
      select.empty();
      select.append('<option value="">Select a model...</option>');
      
      models.forEach(function(model) {
        const name = model.name || model.id;
        const sizeInfo = model.size ? ` (${model.size})` : '';
        const isDefault = model.id === DEFAULT_MODEL;
        const option = `<option value="${model.id}"${isDefault ? ' selected' : ''}>${name}${sizeInfo}</option>`;
        select.append(option);
      });
      
      // Mark the lightweight model as recommended
      if (select.find(`option[value="${DEFAULT_MODEL}"]`).length > 0) {
        select.find(`option[value="${DEFAULT_MODEL}"]`).text(
          select.find(`option[value="${DEFAULT_MODEL}"]`).text() + ' (Recommended for testing)'
        );
        console.log('✅ Default model set to Phi-3.5-mini (lightweight)');
      }
      
    } catch (error) {
      console.error('❌ Failed to load models:', error);
      showStatus('Failed to load available models.', 'error');
    }
  }

  async function loadSelectedModel() {
    const selectedModel = $('#model-select').val();
    
    if (!selectedModel) {
      showStatus('Please select a model first.', 'error');
      return;
    }

    try {
      console.log('🔄 Loading model:', selectedModel);
      showModelLoading();
      
      await engineManager.initializeEngine(selectedModel, (progress) => {
        updateProgress(progress);
      });
      
      hideModelLoading();
      showChatSection();
      showStatus(`Model "${selectedModel}" loaded successfully!`, 'success');
      console.log('✅ Model loaded successfully');
      
    } catch (error) {
      console.error('❌ Failed to load model:', error);
      hideModelLoading();
      showStatus('Failed to load model: ' + error.message, 'error');
    }
  }

  function extractContent() {
    console.log('🔍 Starting content extraction...');
    showLoading('Extracting page content...');
    
    try {
      // Get content from the demo sample content section first
      let content = '';
      const sampleContent = document.querySelector('.demo-sample-content');
      
      if (sampleContent) {
        content = sampleContent.innerText || sampleContent.textContent || '';
        console.log('📄 Found sample content:', content.length, 'characters');
      }
      
      // Fallback to main content areas
      if (!content || content.length < 100) {
        const selectors = ['main', '.main-content', '.content', 'article', '#content'];
        for (const selector of selectors) {
          const element = document.querySelector(selector);
          if (element) {
            content = element.innerText || element.textContent || '';
            if (content.length > 100) {
              console.log('📄 Using content from:', selector);
              break;
            }
          }
        }
      }
      
      // Final fallback to body content
      if (!content || content.length < 100) {
        console.log('⚠️ Using body content as fallback');
        content = document.body.innerText || document.body.textContent || '';
      }
      
      // Clean content
      content = content.replace(/\s+/g, ' ').trim();
      
      console.log('✅ Final content length:', content.length);
      console.log('📄 Content preview:', content.substring(0, 200));
      
      if (!content || content.length < 50) {
        hideLoading();
        showStatus('No readable content found on this page.', 'error');
        return;
      }
      
      // Send to backend
      const apiUrl = drupalSettings.webllm_ai.api_endpoints?.extract_content;
      if (!apiUrl) {
        hideLoading();
        showStatus('API endpoint not configured.', 'error');
        return;
      }
      
      $.ajax({
        url: apiUrl,
        method: 'POST',
        contentType: 'application/json',
        data: JSON.stringify({
          page_content: content,
          page_url: window.location.href,
          page_title: document.title
        }),
        success: function(response) {
          hideLoading();
          console.log('✅ Backend response:', response);
          
          if (response.success) {
            extractedContent = response.content;
            
            // Display results
            $('#extracted-content').val(response.content);
            $('#word-count').text(response.word_count || 'N/A');
            $('#char-count').text(response.char_count || 'N/A');
            $('#source-url').text(response.url || window.location.href);
            
            // Show debug info
            $('#debug-content').text(response.content.substring(0, 1000) + '...');
            
            // Show next sections
            $('.content-display-section').show();
            $('.model-selection-section').show();
            
            showStatus('Content extracted successfully!', 'success');
          } else {
            showStatus('Backend error: ' + (response.error || 'Unknown error'), 'error');
          }
        },
        error: function(xhr, status, error) {
          hideLoading();
          console.error('❌ AJAX error:', xhr, status, error);
          
          let errorMsg = 'Network error';
          if (xhr.responseJSON && xhr.responseJSON.error) {
            errorMsg = xhr.responseJSON.error;
          } else if (xhr.responseText) {
            errorMsg = 'Server error: ' + xhr.status;
          }
          
          showStatus('Error: ' + errorMsg, 'error');
        }
      });
      
    } catch (error) {
      hideLoading();
      console.error('❌ Exception:', error);
      showStatus('Error during extraction: ' + error.message, 'error');
    }
  }

  async function askQuestion() {
    const question = $('#question-input').val().trim();
    
    if (!question) {
      showStatus('Please enter a question.', 'error');
      return;
    }

    if (!engineManager || !engineManager.isReady()) {
      showStatus('Please load a model first.', 'error');
      return;
    }

    if (!extractedContent) {
      showStatus('Please extract page content first.', 'error');
      return;
    }

    try {
      console.log('💭 Processing question:', question);
      
      // Add user message to chat
      addMessageToChat('user', question);
      $('#question-input').val('');
      
      // Show thinking indicator
      const thinkingId = addMessageToChat('assistant', 'Thinking...');
      
      // Create RAG prompt
      console.log('🔄 Creating RAG prompt...');
      const ragPromptResponse = await $.ajax({
        url: drupalSettings.webllm_ai.api_endpoints.process_rag,
        method: 'POST',
        contentType: 'application/json',
        data: JSON.stringify({
          query: question,
          context: extractedContent
        })
      });

      if (!ragPromptResponse.success) {
        throw new Error(ragPromptResponse.error);
      }

      console.log('🤖 Generating AI response...');
      
      // Generate response using WebLLM
      const messages = [{ role: 'user', content: ragPromptResponse.prompt }];
      const response = await engineManager.generateCompletion(messages, {
        temperature: parseFloat(drupalSettings.webllm_ai.temperature) || 0.7,
        max_tokens: parseInt(drupalSettings.webllm_ai.max_tokens) || 1000,
      });

      // Update the assistant message
      const assistantMessage = response.choices[0].message.content;
      updateChatMessage(thinkingId, assistantMessage);
      
      console.log('✅ AI response generated successfully');
      
    } catch (error) {
      console.error('❌ Error generating response:', error);
      updateChatMessage(thinkingId, 'Sorry, I encountered an error: ' + error.message);
      showStatus('Error generating response: ' + error.message, 'error');
    }
  }

  function addMessageToChat(role, content) {
    const messageId = 'msg-' + Date.now() + '-' + Math.random().toString(36).substr(2, 9);
    const messageClass = role === 'user' ? 'user-message' : 'assistant-message';
    
    const messageHtml = `
      <div class="chat-message ${messageClass}" id="${messageId}">
        <div class="message-role">${role === 'user' ? 'You' : 'AI Assistant'}</div>
        <div class="message-content">${escapeHtml(content)}</div>
        <div class="message-time">${new Date().toLocaleTimeString()}</div>
      </div>
    `;
    
    $('#chat-messages').append(messageHtml);
    $('#chat-messages').scrollTop($('#chat-messages')[0].scrollHeight);
    
    return messageId;
  }

  function updateChatMessage(messageId, content) {
    $(`#${messageId} .message-content`).html(escapeHtml(content));
    $('#chat-messages').scrollTop($('#chat-messages')[0].scrollHeight);
  }

  function updateProgress(progress) {
    const percentage = Math.round(progress.progress * 100);
    $('#progress-bar').css('width', percentage + '%');
    $('#progress-text').text(percentage + '%');
    $('#loading-status').text(progress.text || 'Loading...');
  }

  function showModelLoading() {
    $('#loading-progress').show();
    $('#load-model-btn').prop('disabled', true).text('Loading...');
  }

  function hideModelLoading() {
    $('#loading-progress').hide();
    $('#load-model-btn').prop('disabled', false).text('Load Model');
  }

  function showChatSection() {
    $('.chat-section').show();
    $('#question-input').focus();
  }

  function showStatus(message, type) {
    const alertClass = {
      'success': 'alert-success',
      'error': 'alert-danger', 
      'warning': 'alert-warning',
      'info': 'alert-info'
    }[type] || 'alert-info';

    const html = `
      <div class="alert ${alertClass} alert-dismissible" role="alert">
        ${escapeHtml(message)}
        <button type="button" class="btn-close" onclick="$(this).parent().remove()"></button>
      </div>
    `;

    $('#status-messages').append(html);
    
    // Auto remove after 5 seconds
    setTimeout(() => {
      $('#status-messages .alert').first().fadeOut(() => {
        $(this).remove();
      });
    }, 5000);
  }

  function showLoading(message) {
    $('#loading-overlay .loading-text').text(message);
    $('#loading-overlay').show();
  }

  function hideLoading() {
    $('#loading-overlay').hide();
  }

  function escapeHtml(text) {
    const div = document.createElement('div');
    div.textContent = text;
    return div.innerHTML;
  }

})(jQuery, Drupal, drupalSettings);
