// Simplified WebLLM Integration - Focus on getting one model working
console.log('WebLLM Simple Integration Loading...');

let webllmEngine = null;
let isModelLoaded = false;

// Simple status update function
function updateStatus(message) {
    console.log('Status:', message);
    const statusElement = document.querySelector('.status-text');
    if (statusElement) {
        statusElement.textContent = message;
    }
}

// Check WebGPU support
function checkWebGPUSupport() {
    const webgpuElement = document.querySelector('.webgpu-text');
    if ('gpu' in navigator) {
        console.log('✓ WebGPU supported');
        if (webgpuElement) webgpuElement.textContent = 'WebGPU supported ✓';
        return true;
    } else {
        console.log('✗ WebGPU not supported');
        if (webgpuElement) webgpuElement.textContent = 'WebGPU not supported ✗';
        return false;
    }
}

// Load a single hardcoded model for testing
async function loadSimpleModel() {
    const loadBtn = document.getElementById('load-model-btn');
    const progressDiv = document.getElementById('model-progress');
    const progressFill = document.getElementById('progress-fill');
    const progressText = document.getElementById('progress-text');

    try {
        // Use a lightweight model for testing
        const modelId = "Phi-3.5-mini-instruct-q4f16_1-MLC";

        console.log('Loading model:', modelId);
        updateStatus('Loading model: ' + modelId);

        // Show progress
        if (progressDiv) progressDiv.style.display = 'block';
        if (loadBtn) {
            loadBtn.disabled = true;
            loadBtn.textContent = 'Loading...';
        }

        // Progress callback
        const progressCallback = (report) => {
            console.log('Progress:', report);
            if (report.progress !== undefined) {
                const percentage = Math.round(report.progress * 100);
                if (progressFill) progressFill.style.width = percentage + '%';
                if (progressText) progressText.textContent = percentage + '%';
                updateStatus(`Loading: ${percentage}%`);
            }
        };

        // Load the model
        await webllmEngine.reload(modelId, undefined, {
            initProgressCallback: progressCallback
        });

        // Success!
        isModelLoaded = true;
        console.log('✓ Model loaded successfully!');
        updateStatus('Model ready! Try sending a message.');

        if (loadBtn) {
            loadBtn.textContent = 'Model Loaded ✓';
            loadBtn.style.background = '#4caf50';
        }
        if (progressDiv) progressDiv.style.display = 'none';

        // Enable chat
        enableChat();

    } catch (error) {
        console.error('✗ Model loading failed:', error);
        updateStatus('Failed to load model: ' + error.message);

        if (loadBtn) {
            loadBtn.disabled = false;
            loadBtn.textContent = 'Load Model';
        }
        if (progressDiv) progressDiv.style.display = 'none';
    }
}

// Enable chat interface
function enableChat() {
    const chatInput = document.getElementById('chat-input');
    const sendBtn = document.getElementById('send-btn');

    if (chatInput) {
        chatInput.disabled = false;
        chatInput.placeholder = 'Type your message here...';
    }
    if (sendBtn) {
        sendBtn.disabled = false;
    }
}

// Send a message to the model
async function sendMessage() {
    if (!isModelLoaded) {
        alert('Please load a model first');
        return;
    }

    const chatInput = document.getElementById('chat-input');
    const message = chatInput ? chatInput.value.trim() : '';

    if (!message) return;

    console.log('Sending message:', message);

    // Add user message to chat
    addMessage('user', message);
    chatInput.value = '';

    // Disable input while generating
    chatInput.disabled = true;
    const sendBtn = document.getElementById('send-btn');
    if (sendBtn) sendBtn.disabled = true;

    updateStatus('Generating response...');

    try {
        // Generate response
        const response = await webllmEngine.chat.completions.create({
            messages: [{ role: 'user', content: message }],
            temperature: 0.7,
            max_tokens: 500,
        });

        const aiResponse = response.choices[0].message.content;
        console.log('AI response:', aiResponse);

        // Add AI response to chat
        addMessage('assistant', aiResponse);
        updateStatus('Ready for next message');

    } catch (error) {
        console.error('Generation failed:', error);
        addMessage('system', 'Error: ' + error.message);
        updateStatus('Error occurred');
    }

    // Re-enable input
    chatInput.disabled = false;
    if (sendBtn) sendBtn.disabled = false;
    chatInput.focus();
}

// Add message to chat display with enhanced formatting
function addMessage(role, content) {
    const chatMessages = document.getElementById('chat-messages');
    if (!chatMessages) return;

    const messageDiv = document.createElement('div');
    messageDiv.className = `message ${role}`;

    const contentDiv = document.createElement('div');
    contentDiv.className = 'message-content';

    // Enhanced content formatting for better display
    let formattedContent = content;

    // Format code blocks with proper highlighting
    formattedContent = formattedContent.replace(/```(\w+)?\n?([\s\S]*?)```/g, (match, lang, code) => {
        return `<pre><code class="language-${lang || 'text'}">${escapeHtml(code.trim())}</code></pre>`;
    });

    // Format inline code
    formattedContent = formattedContent.replace(/`([^`]+)`/g, '<code>$1</code>');

    // Format line breaks properly
    formattedContent = formattedContent.replace(/\n/g, '<br>');

    // Set content with HTML formatting
    contentDiv.innerHTML = formattedContent;

    messageDiv.appendChild(contentDiv);
    chatMessages.appendChild(messageDiv);

    // Smooth scroll to bottom with animation
    chatMessages.scrollTo({
        top: chatMessages.scrollHeight,
        behavior: 'smooth'
    });

    console.log(`Added ${role} message:`, content.substring(0, 50) + '...');
}

// Helper function to escape HTML
function escapeHtml(text) {
    const div = document.createElement('div');
    div.textContent = text;
    return div.innerHTML;
}

// Initialize everything
document.addEventListener('DOMContentLoaded', async function () {
    console.log('DOM ready - initializing WebLLM...');

    // Check WebGPU first
    checkWebGPUSupport();

    try {
        updateStatus('Loading WebLLM library...');

        // Load WebLLM from CDN
        const webllm = await import('https://esm.run/@mlc-ai/web-llm');
        console.log('✓ WebLLM library loaded');

        // Create engine
        updateStatus('Creating WebLLM engine...');
        webllmEngine = new webllm.MLCEngine();
        console.log('✓ WebLLM engine created'); updateStatus('Ready! Click "Load Model" to start.');        // Set up Load Model button (hardcoded model)
        const loadBtn = document.getElementById('load-model-btn');
        if (loadBtn) {
            loadBtn.disabled = false;
            loadBtn.addEventListener('click', loadSimpleModel);
        }

        // Set up chat interface
        const sendBtn = document.getElementById('send-btn');
        const chatInput = document.getElementById('chat-input');

        if (sendBtn) {
            sendBtn.addEventListener('click', sendMessage);
        }

        if (chatInput) {
            chatInput.addEventListener('keypress', function (e) {
                if (e.key === 'Enter' && !e.shiftKey) {
                    e.preventDefault();
                    sendMessage();
                }
            });
        }

    } catch (error) {
        console.error('✗ Initialization failed:', error);
        updateStatus('Failed to initialize WebLLM: ' + error.message);

        // Show detailed error
        const container = document.querySelector('.webllm-chat-container');
        if (container) {
            const errorDiv = document.createElement('div');
            errorDiv.className = 'error-message';
            errorDiv.innerHTML = `
                <h4>WebLLM Failed to Load</h4>
                <p><strong>Error:</strong> ${error.message}</p>
                <p><strong>Troubleshooting:</strong></p>
                <ul>
                    <li>Make sure you're using Chrome, Firefox, or Edge</li>
                    <li>Check that WebGPU is enabled in your browser</li>
                    <li>Ensure you have a stable internet connection</li>
                    <li>Try refreshing the page</li>
                </ul>
            `;
            container.insertBefore(errorDiv, container.firstChild);
        }
    }
});

console.log('WebLLM Simple Integration script loaded');
