<?php

namespace Drupal\webllm_ai\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\webllm_ai\Service\WebLlmService;
use Drupal\webllm_ai\Service\ModelManager;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * WebLLM AI settings form.
 */
class WebLlmSettingsForm extends ConfigFormBase {

  /**
   * The WebLLM service.
   *
   * @var \Drupal\webllm_ai\Service\WebLlmService
   */
  protected $webllmService;

  /**
   * The model manager.
   *
   * @var \Drupal\webllm_ai\Service\ModelManager
   */
  protected $modelManager;

  /**
   * Constructs a WebLlmSettingsForm object.
   *
   * @param \Drupal\webllm_ai\Service\WebLlmService $webllm_service
   *   The WebLLM service.
   * @param \Drupal\webllm_ai\Service\ModelManager $model_manager
   *   The model manager.
   */
  public function __construct(WebLlmService $webllm_service, ModelManager $model_manager) {
    $this->webllmService = $webllm_service;
    $this->modelManager = $model_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('webllm_ai.service'),
      $container->get('webllm_ai.model_manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['webllm_ai.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'webllm_ai_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('webllm_ai.settings');
    $all_models = $this->modelManager->getAllModels();

    $form['intro'] = [
      '#type' => 'markup',
      '#markup' => '<p>' . $this->t('Configure WebLLM AI settings. WebLLM runs Large Language Models directly in the browser without requiring API keys or external services.') . '</p>',
    ];

    $form['browser_requirements'] = [
      '#type' => 'details',
      '#title' => $this->t('Browser Requirements'),
      '#open' => FALSE,
    ];

    $form['browser_requirements']['info'] = [
      '#type' => 'markup',
      '#markup' => '<ul>' .
        '<li>' . $this->t('Modern browser with WebGPU support (Chrome 113+, Firefox 120+, Safari 16.4+)') . '</li>' .
        '<li>' . $this->t('Sufficient RAM (8GB+ recommended for larger models)') . '</li>' .
        '<li>' . $this->t('Good internet connection for initial model download') . '</li>' .
        '<li>' . $this->t('WebGPU enabled in browser settings') . '</li>' .
        '</ul>',
    ];

    // General settings
    $form['general'] = [
      '#type' => 'details',
      '#title' => $this->t('General Settings'),
      '#open' => TRUE,
    ];

    $form['general']['system_prompt'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Default System Prompt'),
      '#description' => $this->t('The default system prompt that will be used to instruct the AI. This sets the AI\'s behavior and personality.'),
      '#default_value' => $config->get('system_prompt'),
      '#rows' => 4,
      '#required' => TRUE,
    ];

    // Model configuration
    $form['models'] = [
      '#type' => 'details',
      '#title' => $this->t('Model Configuration'),
      '#open' => TRUE,
    ];

    $model_options = [];
    foreach ($all_models as $model_id => $model_info) {
      $model_options[$model_id] = $model_info['name'] . ' (' . $model_info['size'] . ') - ' . $model_info['description'];
    }

    $form['models']['default_model'] = [
      '#type' => 'select',
      '#title' => $this->t('Default Model'),
      '#description' => $this->t('The model that will be selected by default in chat interfaces.'),
      '#options' => $model_options,
      '#default_value' => $config->get('default_model'),
      '#required' => TRUE,
    ];

    $form['models']['allowed_models'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Allowed Models'),
      '#description' => $this->t('Select which models users are allowed to use. Larger models require more memory and take longer to load.'),
      '#options' => $model_options,
      '#default_value' => $config->get('allowed_models') ?: array_keys($model_options),
    ];

    // Model information table
    $form['models']['model_info'] = [
      '#type' => 'details',
      '#title' => $this->t('Model Information'),
      '#open' => FALSE,
    ];

    $header = [
      $this->t('Model'),
      $this->t('Size'),
      $this->t('Memory'),
      $this->t('Performance'),
      $this->t('Languages'),
      $this->t('Description'),
    ];

    $rows = [];
    foreach ($all_models as $model_id => $model_info) {
      $rows[] = [
        $model_info['name'],
        $model_info['size'],
        round($model_info['recommended_memory'] / 1024, 1) . ' GB',
        ucfirst($model_info['performance_tier']),
        implode(', ', array_slice($model_info['languages'], 0, 3)) . (count($model_info['languages']) > 3 ? '...' : ''),
        $model_info['description'],
      ];
    }

    $form['models']['model_info']['table'] = [
      '#type' => 'table',
      '#header' => $header,
      '#rows' => $rows,
      '#empty' => $this->t('No models available.'),
    ];

    // Generation parameters
    $form['parameters'] = [
      '#type' => 'details',
      '#title' => $this->t('Generation Parameters'),
      '#open' => TRUE,
    ];

    $form['parameters']['temperature'] = [
      '#type' => 'number',
      '#title' => $this->t('Temperature'),
      '#description' => $this->t('Controls randomness in responses. Lower values (0.1-0.5) make responses more focused and deterministic. Higher values (0.7-1.0) make responses more creative and varied.'),
      '#default_value' => $config->get('temperature'),
      '#min' => 0,
      '#max' => 2,
      '#step' => 0.1,
      '#required' => TRUE,
      '#element_validate' => ['webllm_ai_validate_temperature'],
    ];

    $form['parameters']['max_tokens'] = [
      '#type' => 'number',
      '#title' => $this->t('Max Tokens'),
      '#description' => $this->t('Maximum number of tokens (words/characters) in the AI response. Higher values allow longer responses but take more time and memory.'),
      '#default_value' => $config->get('max_tokens'),
      '#min' => 1,
      '#max' => 4096,
      '#step' => 1,
      '#required' => TRUE,
      '#element_validate' => ['webllm_ai_validate_max_tokens'],
    ];

    // Performance settings
    $form['performance'] = [
      '#type' => 'details',
      '#title' => $this->t('Performance & Caching'),
      '#open' => FALSE,
    ];

    $form['performance']['use_cdn'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Use CDN for WebLLM library'),
      '#description' => $this->t('Load the WebLLM library from CDN. Recommended for most installations.'),
      '#default_value' => $config->get('use_cdn'),
    ];

    $form['performance']['enable_caching'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable model caching'),
      '#description' => $this->t('Cache downloaded models in the browser for faster subsequent loads.'),
      '#default_value' => $config->get('enable_caching'),
    ];

    $form['performance']['cache_quota_mb'] = [
      '#type' => 'number',
      '#title' => $this->t('Cache quota (MB)'),
      '#description' => $this->t('Maximum amount of storage space to use for model caching.'),
      '#default_value' => $config->get('cache_quota_mb'),
      '#min' => 512,
      '#max' => 51200,
      '#step' => 512,
      '#states' => [
        'visible' => [
          ':input[name="enable_caching"]' => ['checked' => TRUE],
        ],
      ],
    ];

    // Advanced settings
    $form['advanced'] = [
      '#type' => 'details',
      '#title' => $this->t('Advanced Settings'),
      '#open' => FALSE,
    ];

    $form['advanced']['debug_mode'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable debug mode'),
      '#description' => $this->t('Show additional debugging information in browser console.'),
      '#default_value' => $config->get('debug_mode'),
    ];

    $form['advanced']['log_interactions'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Log AI interactions'),
      '#description' => $this->t('Log AI interactions for analytics and debugging. No sensitive data is stored.'),
      '#default_value' => $config->get('log_interactions'),
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    parent::validateForm($form, $form_state);

    // Validate that at least one model is selected
    $allowed_models = array_filter($form_state->getValue('allowed_models'));
    if (empty($allowed_models)) {
      $form_state->setErrorByName('allowed_models', $this->t('At least one model must be allowed.'));
    }

    // Validate that default model is in allowed models
    $default_model = $form_state->getValue('default_model');
    if (!in_array($default_model, $allowed_models)) {
      $form_state->setErrorByName('default_model', $this->t('The default model must be in the list of allowed models.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // Filter out unchecked models
    $allowed_models = array_values(array_filter($form_state->getValue('allowed_models')));

    $this->config('webllm_ai.settings')
      ->set('system_prompt', $form_state->getValue('system_prompt'))
      ->set('default_model', $form_state->getValue('default_model'))
      ->set('allowed_models', $allowed_models)
      ->set('temperature', (float) $form_state->getValue('temperature'))
      ->set('max_tokens', (int) $form_state->getValue('max_tokens'))
      ->set('use_cdn', (bool) $form_state->getValue('use_cdn'))
      ->set('enable_caching', (bool) $form_state->getValue('enable_caching'))
      ->set('cache_quota_mb', (int) $form_state->getValue('cache_quota_mb'))
      ->set('debug_mode', (bool) $form_state->getValue('debug_mode'))
      ->set('log_interactions', (bool) $form_state->getValue('log_interactions'))
      ->save();

    parent::submitForm($form, $form_state);
  }

}
