<?php

namespace Drupal\webllm_ai\Service;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Cache\CacheBackendInterface;

/**
 * WebLLM AI service for managing AI operations.
 */
class WebLlmService {

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The logger factory.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * The cache backend.
   *
   * @var \Drupal\Core\Cache\CacheBackendInterface
   */
  protected $cache;

  /**
   * Constructs a WebLlmService object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory.
   * @param \Drupal\Core\Cache\CacheBackendInterface $cache
   *   The cache backend.
   */
  public function __construct(ConfigFactoryInterface $config_factory, LoggerChannelFactoryInterface $logger_factory, CacheBackendInterface $cache) {
    $this->configFactory = $config_factory;
    $this->logger = $logger_factory->get('webllm_ai');
    $this->cache = $cache;
  }

  /**
   * Get WebLLM AI configuration.
   *
   * @return \Drupal\Core\Config\ImmutableConfig
   *   The configuration object.
   */
  public function getConfig() {
    return $this->configFactory->get('webllm_ai.settings');
  }

  /**
   * Get available models.
   *
   * @return array
   *   Array of available models.
   */
  public function getAvailableModels() {
    $config = $this->getConfig();
    $allowed_models = $config->get('allowed_models') ?: [];
    
    $all_models = [
      'Llama-3.1-8B-Instruct-q4f32_1-MLC' => [
        'name' => 'Llama 3.1 8B',
        'description' => 'Recommended general-purpose model',
        'size' => '8B parameters',
        'memory_usage' => '~4.5GB',
        'performance' => 'Medium',
      ],
      'Llama-3.1-70B-Instruct-q4f16_1-MLC' => [
        'name' => 'Llama 3.1 70B',
        'description' => 'High-quality large model',
        'size' => '70B parameters',
        'memory_usage' => '~35GB',
        'performance' => 'High',
      ],
      'Phi-3.5-mini-instruct-q4f16_1-MLC' => [
        'name' => 'Phi 3.5 Mini',
        'description' => 'Fast and lightweight model',
        'size' => '3.8B parameters',
        'memory_usage' => '~2.2GB',
        'performance' => 'Fast',
      ],
      'Qwen2.5-7B-Instruct-q4f16_1-MLC' => [
        'name' => 'Qwen 2.5 7B',
        'description' => 'Multilingual capable model',
        'size' => '7B parameters',
        'memory_usage' => '~4GB',
        'performance' => 'Medium',
      ],
      'Qwen2.5-14B-Instruct-q4f16_1-MLC' => [
        'name' => 'Qwen 2.5 14B',
        'description' => 'Larger multilingual model',
        'size' => '14B parameters',
        'memory_usage' => '~8GB',
        'performance' => 'Medium-High',
      ],
      'Gemma-2-9B-It-q4f16_1-MLC' => [
        'name' => 'Gemma 2 9B',
        'description' => 'Google Gemma model',
        'size' => '9B parameters',
        'memory_usage' => '~5GB',
        'performance' => 'Medium',
      ],
      'Mistral-7B-Instruct-v0.3-q4f16_1-MLC' => [
        'name' => 'Mistral 7B v0.3',
        'description' => 'Efficient instruction-following model',
        'size' => '7B parameters',
        'memory_usage' => '~4GB',
        'performance' => 'Medium',
      ],
    ];

    // Filter models based on allowed list
    $available_models = [];
    foreach ($allowed_models as $model_id) {
      if (isset($all_models[$model_id])) {
        $available_models[$model_id] = $all_models[$model_id];
      }
    }

    return $available_models;
  }

  /**
   * Get the default model ID.
   *
   * @return string
   *   The default model ID.
   */
  public function getDefaultModel() {
    return $this->getConfig()->get('default_model') ?: 'Llama-3.1-8B-Instruct-q4f32_1-MLC';
  }

  /**
   * Get the default system prompt.
   *
   * @return string
   *   The default system prompt.
   */
  public function getDefaultSystemPrompt() {
    return $this->getConfig()->get('system_prompt') ?: 'You are a helpful AI assistant.';
  }

  /**
   * Get generation parameters.
   *
   * @return array
   *   Array of generation parameters.
   */
  public function getGenerationParameters() {
    $config = $this->getConfig();
    return [
      'temperature' => (float) ($config->get('temperature') ?: 0.7),
      'max_tokens' => (int) ($config->get('max_tokens') ?: 1000),
    ];
  }

  /**
   * Validate a model ID.
   *
   * @param string $model_id
   *   The model ID to validate.
   *
   * @return bool
   *   TRUE if the model is valid and allowed.
   */
  public function isValidModel($model_id) {
    $available_models = $this->getAvailableModels();
    return isset($available_models[$model_id]);
  }

  /**
   * Log an AI interaction.
   *
   * @param string $model_id
   *   The model used.
   * @param string $prompt
   *   The user prompt.
   * @param string $response
   *   The AI response.
   * @param array $metadata
   *   Additional metadata.
   */
  public function logInteraction($model_id, $prompt, $response, array $metadata = []) {
    $this->logger->info('WebLLM AI interaction: Model @model, Prompt length: @prompt_length, Response length: @response_length', [
      '@model' => $model_id,
      '@prompt_length' => strlen($prompt),
      '@response_length' => strlen($response),
    ]);

    // Store in cache for potential analytics
    $cache_key = 'webllm_ai_interaction_' . time() . '_' . uniqid();
    $cache_data = [
      'model_id' => $model_id,
      'prompt_length' => strlen($prompt),
      'response_length' => strlen($response),
      'timestamp' => time(),
      'metadata' => $metadata,
    ];
    
    $this->cache->set($cache_key, $cache_data, time() + 86400); // Cache for 24 hours
  }

  /**
   * Get usage statistics.
   *
   * @return array
   *   Array of usage statistics.
   */
  public function getUsageStatistics() {
    // This would typically query a more permanent storage
    // For now, we'll return basic stats
    return [
      'total_interactions' => 0,
      'popular_models' => [],
      'average_response_time' => 0,
    ];
  }

  /**
   * Clean up old cached data.
   */
  public function cleanupCache() {
    // Implement cache cleanup logic
    $this->logger->info('WebLLM AI cache cleanup completed');
  }

  /**
   * Check browser compatibility for WebLLM.
   *
   * @param string $user_agent
   *   The user agent string.
   *
   * @return array
   *   Compatibility information.
   */
  public function checkBrowserCompatibility($user_agent = NULL) {
    // Basic browser compatibility check
    // WebLLM requires WebGPU support
    return [
      'compatible' => TRUE, // We'll assume compatibility for now
      'webgpu_support' => 'unknown',
      'recommendations' => [
        'Use a modern browser (Chrome 113+, Firefox 120+, Safari 16.4+)',
        'Ensure WebGPU is enabled',
        'Have sufficient RAM (8GB+ recommended)',
      ],
    ];
  }

}
